"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Part of the default module.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from .module_core import ModuleCore


def get_name():
    """
    Readable module name, for example, 'Default Module'.
    """
    return ModuleCore.module_name


def is_available() -> bool:
    """
    Check for any additional requirements for the module here.
    """
    return True



# Hindi lexemes settings_dialog.py
lexemes = {
    "tab_openai_api_config": "OpenAI API",

    "module_openai_api_label": "OpenAI API",
    "module_openai_api_url_label": "API URL",
    "module_openai_api_url_input_placeholder_text": "API URL",
    "module_openai_api_url_input_accessible_description":
        "OpenAI API का URL API के अंतिम बिंदु का पता है, जो सेवा और संस्करण के आधार पर भिन्न हो सकता है।\n"
        "AI सहायक इसका उपयोग वार्तालाप चैट या पाठ पूर्णता के लिए करता है।\n"
        "वर्तमान URL प्राप्त करने के लिए OpenAI API के आधिकारिक दस्तावेज़ का संदर्भ लें।",
    "module_openai_api_key_label": "API कुंजी",
    "module_openai_api_key_input_placeholder_text": "API कुंजी",
    "module_openai_api_key_input_accessible_description":
        "OpenAI API कुंजी API के अंतिम बिंदु पर अनुरोधों को प्रमाणित करने के लिए उपयोग किया जाने वाला एक गुप्त टोकन है।",
    "module_openai_api_supported_models_label": "समर्थित मॉडल",
    "module_openai_api_model_names_combo_placeholder_text": "एक मॉडल चुनें",
    "module_openai_api_model_names_combo_accessible_description":
        "चैट वार्तालापों के लिए समर्थित मॉडलों में से चुनें।",

    "module_openai_api_base_system_prompt_label": "सिस्टम प्रॉम्प्ट",
    "module_openai_api_base_system_prompt_edit_placeholder_text": "प्रत्येक अनुरोध से पहले आधार सिस्टम प्रॉम्प्ट",
    "module_openai_api_base_system_prompt_edit_accessible_description":
        "प्रत्येक अनुरोध से पहले आने वाला आधार सिस्टम प्रॉम्प्ट।\n"
        "आमतौर पर, यह निर्देशों या भूमिका विशेषताओं वाला सादा पाठ होता है।",

    "module_openai_api_base_response_temperature_label": "तापमान: {temperature}",
    "module_openai_api_base_response_temperature_input_accessible_description":
        "मॉडल की प्रतिक्रियाओं की यादृच्छिकता को समायोजित करता है। उच्च मूल्य अधिक विविधतापूर्ण\n"
        "आउटपुट उत्पन्न करते हैं, जबकि निम्न मूल्य अधिक पूर्वानुमान योग्य प्रतिक्रियाएं देते हैं।",

    "module_openai_api_base_response_max_tokens_label": "अधिकतम प्रतिक्रिया टोकन",
    "module_openai_api_base_response_max_tokens_input_accessible_description":
        "प्रतिक्रिया में प्राप्त होने वाले टोकनों की अधिकतम संख्या निर्धारित करता है, जैसे शब्द और विराम चिह्न,\n"
        "आउटपुट की लंबाई को नियंत्रित करता है।",

    "module_openai_api_config_prompt_history_size_label": "प्रॉम्प्ट इतिहास का आकार",
    "module_openai_api_config_prompt_history_size_input_accessible_description":
        "प्रणाली द्वारा संदर्भ के लिए बनाए रखी गई प्रॉम्प्ट इतिहास की प्रविष्टियों की संख्या को नियंत्रित करता है।\n"
        "शून्य मूल्य असीमित प्रविष्टियों की अनुमति देता है।"
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import Qt
from PySide6.QtTest import QTest

from notolog.notolog_editor import NotologEditor
from notolog.settings import Settings
from notolog.enums.languages import Languages

from . import test_app  # noqa: F401

import pytest


class TestQtUi:

    @pytest.fixture(scope="class", autouse=True)
    def settings_obj(self):
        """
        Use 'autouse=True' to enable automatic setup, or pass 'settings_obj' directly to main_window()
        """
        # Fixture to create and return settings instance
        settings = Settings()
        # Clear settings to be sure start over without side effects
        settings.clear()
        # Reset singleton (qa functionality)
        Settings.reload()

        yield settings

    @pytest.fixture
    def main_window(self, mocker, test_app):  # noqa: F811 redefinition of unused 'test_app'
        # Force to override system language as a default
        mocker.patch.object(Languages, 'default', return_value='la')

        # Do not show actual window; return object instance only
        mocker.patch.object(NotologEditor, 'show', return_value=None)
        # Prevent resource processing, including 'process_document_images'
        mocker.patch.object(NotologEditor, 'load_content_html', return_value=None)

        # Fixture to create and return main window instance
        yield NotologEditor(screen=test_app.screens()[0])

    def test_editor_state(self, main_window, settings_obj):
        # Check app language set correctly
        assert settings_obj.app_language == 'la'

        # Check default window title
        assert main_window.windowTitle() == "Editorium Notolog"

        assert main_window.statusbar['mode_label'].text() == 'Modus Visum'
        assert main_window.statusbar['source_label'].text() == 'HTML'
        assert main_window.statusbar['encryption_label'].text() == 'Simplicitas 🔓'

        # Test that clicking the edit button updates the editor state
        button = main_window.toolbar.toolbar_edit_button
        # assert button
        QTest.mouseClick(button, Qt.MouseButton.LeftButton, pos=button.rect().center())

        assert main_window.statusbar['mode_label'].text() == 'Modus Editio'
        assert main_window.statusbar['source_label'].text() == 'Markdown'
        assert main_window.statusbar['encryption_label'].text() == 'Simplicitas 🔓'

    def test_search_elements(self, main_window):
        assert hasattr(main_window.toolbar, 'search_form')
        main_window.toolbar.search_form.set_text('Test search')
        assert main_window.toolbar.search_form.text() == 'Test search'

        assert hasattr(main_window.toolbar.search_form, 'btn_search_clear')
        # Test that clicking the button updates the search field
        QTest.mouseClick(main_window.toolbar.search_form.btn_search_clear, Qt.MouseButton.LeftButton)  # noqa
        assert main_window.toolbar.search_form.text() == ''



# Japanese lexemes color_picker_dialog.py
lexemes = {
    "color_picker_title": "色を選択",

    "color_picker_color_black": "黒",
    "color_picker_color_silver": "銀色",
    "color_picker_color_gray": "灰色",
    "color_picker_color_white": "白",
    "color_picker_color_maroon": "栗色",
    "color_picker_color_red": "赤",
    "color_picker_color_purple": "紫",
    "color_picker_color_fuchsia": "紫紅色",
    "color_picker_color_green": "緑",
    "color_picker_color_lime": "ライム",
    "color_picker_color_olive": "オリーブ",
    "color_picker_color_yellow": "黄色",
    "color_picker_color_navy": "ネイビー",
    "color_picker_color_blue": "青",
    "color_picker_color_teal": "ティール",
    "color_picker_color_aqua": "アクア",
    "color_picker_color_aliceblue": "アリスブルー",
    "color_picker_color_antiquewhite": "アンティークホワイト",
    "color_picker_color_aquamarine": "アクアマリン",
    "color_picker_color_azure": "空色",
    "color_picker_color_beige": "ベージュ",
    "color_picker_color_bisque": "ビスク",
    "color_picker_color_blanchedalmond": "ブランチドアーモンド",
    "color_picker_color_blueviolet": "ブルーバイオレット",
    "color_picker_color_brown": "茶色",
    "color_picker_color_burlywood": "バーリーウッド",
    "color_picker_color_cadetblue": "キャデットブルー",
    "color_picker_color_chartreuse": "シャルトリューズ",
    "color_picker_color_chocolate": "チョコレート",
    "color_picker_color_coral": "コーラル",
    "color_picker_color_cornflowerblue": "コーンフラワーブルー",
    "color_picker_color_cornsilk": "コーンシルク",
    "color_picker_color_crimson": "クリムゾン",
    "color_picker_color_cyan": "シアン",
    "color_picker_color_darkblue": "ダークブルー",
    "color_picker_color_darkcyan": "ダークシアン",
    "color_picker_color_darkgoldenrod": "ダークゴールデンロッド",
    "color_picker_color_darkgray": "ダークグレー",
    "color_picker_color_darkgreen": "ダークグリーン",
    "color_picker_color_darkkhaki": "ダークカーキ",
    "color_picker_color_darkmagenta": "ダークマゼンタ",
    "color_picker_color_darkolivegreen": "ダークオリーブグリーン",
    "color_picker_color_darkorange": "ダークオレンジ",
    "color_picker_color_darkorchid": "ダークオーキッド",
    "color_picker_color_darkred": "ダークレッド",
    "color_picker_color_darksalmon": "ダークサーモン",
    "color_picker_color_darkseagreen": "ダークシーグリーン",
    "color_picker_color_darkslateblue": "ダークスレートブルー",
    "color_picker_color_darkslategray": "ダークスレートグレー",
    "color_picker_color_darkturquoise": "ダークターコイズ",
    "color_picker_color_darkviolet": "ダークバイオレット",
    "color_picker_color_deeppink": "ディープピンク",
    "color_picker_color_deepskyblue": "ディープスカイブルー",
    "color_picker_color_dimgray": "ディムグレー",
    "color_picker_color_dodgerblue": "ドジャーブルー",
    "color_picker_color_firebrick": "ファイアーブリック",
    "color_picker_color_floralwhite": "フローラルホワイト",
    "color_picker_color_forestgreen": "フォレストグリーン",
    "color_picker_color_gainsboro": "ゲインズボロ",
    "color_picker_color_ghostwhite": "ゴーストホワイト",
    "color_picker_color_gold": "ゴールド",
    "color_picker_color_goldenrod": "ゴールデンロッド",
    "color_picker_color_greenyellow": "グリーンイエロー",
    "color_picker_color_honeydew": "ハニーデュー",
    "color_picker_color_hotpink": "ホットピンク",
    "color_picker_color_indianred": "インディアンレッド",
    "color_picker_color_indigo": "インディゴ",
    "color_picker_color_ivory": "アイボリー",
    "color_picker_color_khaki": "カーキ",
    "color_picker_color_lavender": "ラベンダー",
    "color_picker_color_lavenderblush": "ラベンダーブラッシュ",
    "color_picker_color_lawngreen": "ローングリーン",
    "color_picker_color_lemonchiffon": "レモンシフォン",
    "color_picker_color_lightblue": "ライトブルー",
    "color_picker_color_lightcoral": "ライトコーラル",
    "color_picker_color_lightcyan": "ライトシアン",
    "color_picker_color_lightgoldenrodyellow": "ライトゴールデンロッドイエロー",
    "color_picker_color_lightgray": "ライトグレー",
    "color_picker_color_lightgreen": "ライトグリーン",
    "color_picker_color_lightpink": "ライトピンク",
    "color_picker_color_lightsalmon": "ライトサーモン",
    "color_picker_color_lightseagreen": "ライトシーグリーン",
    "color_picker_color_lightskyblue": "ライトスカイブルー",
    "color_picker_color_lightslategray": "ライトスレートグレー",
    "color_picker_color_lightsteelblue": "ライトスチールブルー",
    "color_picker_color_lightyellow": "ライトイエロー",
    "color_picker_color_limegreen": "ライムグリーン",
    "color_picker_color_linen": "リネン",
    "color_picker_color_magenta": "マゼンタ",
    "color_picker_color_mediumaquamarine": "ミディアムアクアマリン",
    "color_picker_color_mediumblue": "ミディアムブルー",
    "color_picker_color_mediumorchid": "ミディアムオーキッド",
    "color_picker_color_mediumpurple": "ミディアムパープル",
    "color_picker_color_mediumseagreen": "ミディアムシーグリーン",
    "color_picker_color_mediumslateblue": "ミディアムスレートブルー",
    "color_picker_color_mediumspringgreen": "ミディアムスプリンググリーン",
    "color_picker_color_mediumturquoise": "ミディアムターコイズ",
    "color_picker_color_mediumvioletred": "ミディアムバイオレットレッド",
    "color_picker_color_midnightblue": "ミッドナイトブルー",
    "color_picker_color_mintcream": "ミントクリーム",
    "color_picker_color_mistyrose": "ミスティーローズ",
    "color_picker_color_moccasin": "モカシン",
    "color_picker_color_navajowhite": "ナバホホワイト",
    "color_picker_color_oldlace": "オールドレース",
    "color_picker_color_olivedrab": "オリーブドラブ",
    "color_picker_color_orange": "オレンジ",
    "color_picker_color_orangered": "オレンジレッド",
    "color_picker_color_orchid": "オーキッド",
    "color_picker_color_palegoldenrod": "ペールゴールデンロッド",
    "color_picker_color_palegreen": "ペールグリーン",
    "color_picker_color_paleturquoise": "ペールターコイズ",
    "color_picker_color_palevioletred": "ペールバイオレットレッド",
    "color_picker_color_papayawhip": "パパイヤ",
    "color_picker_color_peachpuff": "ピーチパフ",
    "color_picker_color_peru": "ペルー",
    "color_picker_color_pink": "ピンク",
    "color_picker_color_plum": "プラム",
    "color_picker_color_powderblue": "パウダーブルー",
    "color_picker_color_rebeccapurple": "レベッカパープル",
    "color_picker_color_rosybrown": "ロージーブラウン",
    "color_picker_color_royalblue": "ロイヤルブルー",
    "color_picker_color_saddlebrown": "サドルブラウン",
    "color_picker_color_salmon": "サーモン",
    "color_picker_color_sandybrown": "サンディブラウン",
    "color_picker_color_seagreen": "シーグリーン",
    "color_picker_color_seashell": "シーシェル",
    "color_picker_color_sienna": "シエナ",
    "color_picker_color_skyblue": "スカイブルー",
    "color_picker_color_slateblue": "スレートブルー",
    "color_picker_color_slategray": "スレートグレイ",
    "color_picker_color_snow": "スノー",
    "color_picker_color_springgreen": "スプリンググリーン",
    "color_picker_color_steelblue": "スチールブルー",
    "color_picker_color_tan": "タン",
    "color_picker_color_thistle": "シスル",
    "color_picker_color_tomato": "トマト",
    "color_picker_color_turquoise": "ターコイズ",
    "color_picker_color_violet": "バイオレット",
    "color_picker_color_wheat": "ウィート",
    "color_picker_color_whitesmoke": "ホワイトスモーク",
    "color_picker_color_yellowgreen": "イエローグリーン",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: App update helper class.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import QObject, Signal, Slot, QUrl
from PySide6.QtNetwork import QNetworkAccessManager, QNetworkRequest, QNetworkReply, QSslSocket

from packaging import version
from typing import TYPE_CHECKING

import json
import logging

from . import AppConfig

from ..settings import Settings
from ..lexemes.lexemes import Lexemes

if TYPE_CHECKING:
    from PySide6.QtCore import QByteArray  # noqa: F401


class UpdateHelper(QObject):
    """
    # How to use example:
    update_helper = UpdateHelper()
    update_helper.new_version_available.connect(lambda v:
                                                QMessageBox.information(None, "Update Available",
                                                                        f"A new version {v} is available."))
    update_helper.check_for_updates()
    """

    STATUS_OK = 'ok'
    STATUS_ERROR = 'error'

    # Signal to emit if a new version is available
    new_version_check_response = Signal(object)  # Or: Signal(str)

    def __init__(self):
        super().__init__()

        self.settings = Settings(parent=self)

        self.logger = logging.getLogger('update_helper')

        self.logger.info("SSL supported: %s" % QSslSocket.supportsSsl())

        # Load lexemes for selected language and scope
        self.lexemes = Lexemes(self.settings.app_language, default_scope='common')

        self.current_version = version.parse(AppConfig().get_app_version())

        self.manager = QNetworkAccessManager(self)

    def check_for_updates(self):
        # Updates url
        release_url = AppConfig().get_repository_github_api_release_url()
        request = QNetworkRequest(QUrl(release_url))
        self.logger.info(f'Check for update request to {release_url}')
        # Set request data
        # reply = self.manager.post(request, request_data)  # POST
        reply = self.manager.get(request)  # GET

        # Connect finished signal
        reply.finished.connect(self.handle_network_reply)
        # Connect error signal
        reply.errorOccurred.connect(lambda error: self.handle_network_reply(error))

    @Slot(str)
    def handle_network_reply(self, error_code=None):
        # Get reply object that emitted the signal.
        reply = self.sender()  # type: QNetworkReply

        # Get received status code, say 200
        status_code = reply.attribute(QNetworkRequest.Attribute.HttpStatusCodeAttribute)

        self.logger.debug(f'Update response, reply: {reply}, error_code: {error_code}, status_code: {status_code}')

        # Make sure there is no error
        if reply.error() == QNetworkReply.NetworkError.NoError:
            result_message = self.process_response(reply, status_code)
        elif reply.error() == QNetworkReply.NetworkError.HostNotFoundError:
            # The host was not found, indicating possible DNS issues or no internet connection
            result_message = self.lexemes.get('network_connection_error_connection_or_dns')
        elif reply.error() == QNetworkReply.NetworkError.ConnectionRefusedError:
            # Connection was refused, indicating the server might be down or there are network issues
            result_message = self.lexemes.get('network_connection_error_connection_refused')
        elif reply.error() == QNetworkReply.NetworkError.TimeoutError:
            # The connection timed out, indicating network issues
            result_message = self.lexemes.get('network_connection_error_connection_timed_out')
        elif reply.error() == QNetworkReply.NetworkError.ContentNotFoundError:
            # The requested page or resource is not found
            result_message = self.lexemes.get('network_connection_error_connection_404_error')
        else:
            # Handle other errors
            result_message = self.lexemes.get('network_connection_error_generic_with_status_code',
                                              status_code=status_code)

        if error_code is not None:
            self.logger.warning(result_message)
            self.logger.warning(f"Failed to fetch update information: {reply.errorString()}")
            # Emit error signal
            self.new_version_check_response.emit({'status': self.STATUS_ERROR, 'msg': result_message})

        reply.finished.disconnect()
        reply.errorOccurred.disconnect()
        reply.deleteLater()  # Clean up the QNetworkReply object

    def process_response(self, reply: QNetworkReply, status_code) -> str:
        # The request was successful
        data = reply.readAll()  # type: QByteArray
        # Or like this:
        # data = reply.readAll().data().decode('utf-8')
        # json_data = json.loads(data)

        self.logger.debug(f'Raw RESPONSE [{status_code}]: {data}')

        # json_document = QJsonDocument.fromJson(data)
        # json_data = json_document.object()

        # Convert QByteArray to string
        res_string = data.toStdString()  # Or: str(data.data(), encoding='utf-8')
        self.logger.debug(f'Result multi-line STRING: {res_string}')

        # Clean up the string and make one line
        json_str = ''.join(line.strip() for line in res_string.splitlines())

        self.logger.debug(f'Result STRING: {json_str}')

        result_message = self.lexemes.get('network_connection_error_empty')
        # Parse JSON response
        try:
            json_data = json.loads(json_str)
            self.logger.debug(f"Result JSON: {json_data}")

            latest_version_str = json_data['tag_name']
            latest_version = version.parse(latest_version_str)

            if latest_version > self.current_version:
                # 'releases/latest' will redirect to 'releases/tag/v1.2.3'
                update_link = ('<a href="%s">%s</a>'
                               % (AppConfig().get_repository_github_release_url(), latest_version))
                result_message = self.lexemes.get('update_helper_new_version_is_available',
                                                  latest_version=update_link)
                self.logger.info("New version of the app is available: %s. Current version is %s. Response: %s"
                                 % (latest_version, self.current_version, reply.errorString()))
                # Emit the new version signal
                self.new_version_check_response.emit(
                    {'status': self.STATUS_OK, 'msg': result_message,
                     'current_version': self.current_version,
                     # Check new version by this var
                     'new_version': latest_version_str})
            else:
                result_message = self.lexemes.get('update_helper_latest_version_installed')
                self.logger.info("No new version of the app is available, the current version is %s."
                                 % self.current_version)
                self.logger.debug("Version check response: %s, %s" % (reply.readAll(), reply.errorString()))
                # Emit the same version signal
                self.new_version_check_response.emit(
                    {'status': self.STATUS_OK, 'msg': result_message, 'current_version': self.current_version})

        except json.JSONDecodeError as e:
            self.logger.warning("Error decoding JSON: %s" % e)

        return result_message



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

import asyncio

from PySide6.QtCore import Qt
from PySide6.QtTest import QTest
from PySide6.QtWidgets import QMainWindow

from notolog.ui.ai_assistant.ai_assistant import AIAssistant
from notolog.ui.ai_assistant.ai_message_label import AIMessageLabel
from notolog.settings import Settings
from notolog.enums.languages import Languages
from notolog.modules.base_ai_core import BaseAiCore

from . import test_app  # noqa: F401

from unittest.mock import Mock

import pytest


class TestAiAssistant:

    @pytest.fixture(scope="class", autouse=True)
    def settings_obj(self):
        """
        Use 'autouse=True' to enable automatic setup, or pass 'settings_obj' directly to main_window()
        """
        # Fixture to create and return settings instance
        settings = Settings()
        # Clear settings to be sure start over without side effects
        settings.clear()
        # Reset singleton (qa functionality)
        Settings.reload()

        # Remember to pass the fixture to the other fixtures (check ui_obj())
        yield settings

    @pytest.fixture
    def main_window(self, mocker, test_app):  # noqa: F811 redefinition of unused 'test_app'
        # Force to override system language as a default
        mocker.patch.object(Languages, 'default', return_value='la')

        # Fixture to create and return main window instance
        yield QMainWindow()

    @pytest.fixture(autouse=True)
    def ui_obj(self, settings_obj, main_window):
        # Fixture to initialize object.
        ui_obj = AIAssistant(parent=main_window)
        yield ui_obj

    @pytest.fixture(scope="function")
    def test_exp_params_fixture(self, request):
        """
        Fixture to pass params and get expected results.
        This method is used only for passing params via pytest fixture.
        """
        # Retrieve parameter values from the test request.
        param_values = request.param

        yield param_values

    def test_ui_object_state(self, ui_obj: AIAssistant, settings_obj):
        # Check app language set correctly
        assert settings_obj.app_language == 'la'

        # Check default AI model
        assert ui_obj.inference_module == 'openai_api'

        # Check default window title
        assert ui_obj.windowTitle() == "Postulatio ad AI"

    @pytest.mark.asyncio
    @pytest.mark.parametrize(
        "test_exp_params_fixture",
        [
            "Test Message",
            "Another Test Message",
        ],
        indirect=True
    )
    async def test_add_message(self, mocker, ui_obj: AIAssistant, settings_obj, test_exp_params_fixture):
        # Retrieve callback methods for verification.
        mock_send_request = mocker.patch.object(ui_obj, 'send_request', wraps=ui_obj.send_request)
        mock_update_usage = mocker.patch.object(ui_obj, 'update_usage', wraps=ui_obj.update_usage)

        mock_status_waiting = mocker.patch.object(ui_obj, 'set_status_waiting')
        mock_status_ready = mocker.patch.object(ui_obj, 'set_status_ready')

        module_core = Mock(spec=BaseAiCore)
        mock_init_module = mocker.patch.object(ui_obj, 'init_module', return_value=module_core)
        mock_cancel_request = mocker.patch.object(ui_obj, 'cancel_request', wraps=ui_obj.cancel_request)

        mock_send_request_finished_callback = mocker.patch.object(
            ui_obj, 'send_request_finished_callback', wraps=ui_obj.send_request_finished_callback)

        setattr(ui_obj.settings, 'ai_config_convert_to_md', False)

        mock_request = mocker.patch.object(module_core, 'request', return_value=Mock)

        self.assert_check_message_added = False

        def _check_message_added(message_text):
            assert test_exp_params_fixture in message_text
            self.assert_check_message_added = True

        # Verify the 'message_added' signal
        ui_obj.message_added.connect(_check_message_added)

        assert not self.assert_check_message_added

        # Check the there are no messages in conversation
        messages = ui_obj.messages_area.findChildren(AIMessageLabel)
        assert len(messages) == 0
        # Test that clicking the edit button updates the editor state
        ui_obj.prompt_input.setText(test_exp_params_fixture)
        QTest.mouseClick(ui_obj.send_button, Qt.MouseButton.LeftButton, pos=ui_obj.send_button.rect().center())
        # Check the conversation message was added
        messages = ui_obj.messages_area.findChildren(AIMessageLabel)
        assert len(messages) == 1
        for _message in messages:
            assert isinstance(_message, AIMessageLabel)
            assert _message.text() == test_exp_params_fixture

        # Check the method called
        mock_send_request.assert_called_once()

        # Check these methods were called
        mock_status_waiting.assert_called_once()
        mock_init_module.assert_called_once()

        assert self.assert_check_message_added

        # To allow completion of an async task
        await asyncio.sleep(0.05)

        # Verify that the method was called
        mock_status_ready.assert_not_called()

        # Finished signal with expected results
        # _expected_params = {'message_type': EnumMessageType.RESPONSE, 'request_msg_id': 3, 'response_msg_id': 6}
        # mock_send_request_finished_callback.assert_called_once_with(**_expected_params)
        mock_send_request_finished_callback.assert_not_called()

        # Ensure the method is not called, as it originates from the actual network response handling method.
        mock_update_usage.assert_not_called()

        mock_request.assert_called_once()
        mock_cancel_request.assert_not_called()  # Either an exception or stop button click



# Greek lexemes common.py
lexemes = {
    # Common
    "module_ondevice_llm_name": "LLM στη συσκευή",

    "module_ondevice_llm_model_exception": "Σφάλμα μη εύρεσης μοντέλου: {error_msg}",
    "module_ondevice_llm_task_exception": "Αδυναμία αρχικοποίησης του δημιουργού μοντέλου: {error_msg}",
}



# Greek lexemes settings_dialog.py
lexemes = {
    # Διάλογος ρυθμίσεων
    "window_title": "Ρυθμίσεις",

    "button_close": "Κλείσιμο",

    "tab_general": "Γενικά",
    "tab_editor_config": "Επεξεργαστής",
    "tab_viewer_config": "Προβολέας",
    "tab_ai_config": "Ρυθμίσεις AI",

    "general_app_config_label": "Ρυθμίσεις Εφαρμογής",
    "general_app_language_label": "Γλώσσα",
    "general_app_language_combo_placeholder_text": "Επιλέξτε γλώσσα",
    "general_app_language_combo_accessible_description": "Γλώσσα διεπαφής της εφαρμογής",
    "general_app_theme_label": "Θέμα",
    "general_app_theme_combo_placeholder_text": "Επιλέξτε θέμα",
    "general_app_theme_combo_accessible_description": "Θέμα διεπαφής της εφαρμογής",
    "general_app_main_menu_label": "Κύριο Μενού",
    "general_app_main_menu_checkbox": "Εμφάνιση Κύριου Μενού",
    "general_app_main_menu_checkbox_accessible_description": "Εμφάνιση του κύριου αναδυόμενου μενού της εφαρμογής",
    "general_app_font_size_label": "Μέγεθος Γραμματοσειράς: {size}pt",
    "general_app_font_size_slider_accessible_description":
        "Ρύθμιση του παγκόσμιου μεγέθους γραμματοσειράς της εφαρμογής",

    "general_statusbar_label": "Γραμμή Κατάστασης",
    "general_statusbar_show_global_cursor_position_checkbox": "Εμφάνιση Παγκόσμιας Θέσης Δρομέα",
    "general_statusbar_show_global_cursor_position_checkbox_accessible_description":
        "Εμφάνιση της παγκόσμιας θέσης του δρομέα στη γραμμή κατάστασης",

    "editor_config_label": "Ρυθμίσεις Επεξεργαστή",
    "editor_config_show_line_numbers_checkbox": "Εμφάνιση Αριθμών Γραμμών",
    "editor_config_show_line_numbers_checkbox_accessible_description": "Εμφάνιση αριθμών γραμμών στον επεξεργαστή",

    "viewer_config_label": "Ρυθμίσεις Προβολέα",
    "viewer_config_process_emojis_checkbox": "Μετατροπή Εικονιδίων Κειμένου σε Γραφικά",
    "viewer_config_process_emojis_checkbox_accessible_description":
        "Μετατροπή εικονιδίων κειμένου σε γραφικές αναπαραστάσεις",
    "viewer_config_highlight_todos_checkbox": "Επισήμανση Σημειώσεων TODO",
    "viewer_config_highlight_todos_checkbox_accessible_description": "Επισήμανση ετικετών TODO μέσα στο κείμενο",
    "viewer_config_open_link_confirmation_checkbox": "Απαίτηση Επιβεβαίωσης για Άνοιγμα Συνδέσμων",
    "viewer_config_open_link_confirmation_checkbox_accessible_description":
        "Ζήτηση επιβεβαίωσης πριν το άνοιγμα συνδέσμων",
    "viewer_config_save_resources_checkbox": "Αυτόματη αποθήκευση εξωτερικών εικόνων στον δίσκο",
    "viewer_config_save_resources_checkbox_accessible_description":
        "Αυτόματη αποθήκευση αντιγράφων εξωτερικών εικόνων στον δίσκο για πρόσβαση χωρίς σύνδεση.",

    "ai_config_inference_module_label": "Μονάδα Συμπερασμού",
    "ai_config_inference_module_names_combo_label": "Ενεργή Μονάδα Συμπερασμού",
    "ai_config_inference_module_names_combo_placeholder_text": "Επιλέξτε Μονάδα",
    "ai_config_inference_module_names_combo_accessible_description":
        "Επιλέξτε από τις διαθέσιμες μονάδες συμπερασμού AI για να λειτουργήσετε με τον AI Assistant.\n"
        "Οι επιλογές περιλαμβάνουν μεγάλα γλωσσικά μοντέλα (LLM) με επεξεργασία σε πραγματικό χρόνο,\n"
        "ή λειτουργικότητα βασισμένη σε API.",

    "ai_config_base_label": "Βασικές Παράμετροι",
    "ai_config_multi_turn_dialogue_checkbox": "Πολλαπλός διάλογος με μνήμη συνομιλιών",
    "ai_config_multi_turn_dialogue_checkbox_accessible_description":
        "Ενεργοποιήστε έναν πολλαπλό διάλογο που διατηρεί την προηγούμενη προτροπή για μνήμη συνομιλιών.\n"
        "Όταν είναι απενεργοποιημένο, μόνο το νέο μήνυμα και η προτροπή συστήματος επηρεάζουν την απάντηση.",
    "ai_config_convert_to_md_checkbox": "Μετατροπή του αποτελέσματος σε Markdown",
    "ai_config_convert_to_md_checkbox_accessible_description":
        "Μετατρέψτε το μήνυμα εξόδου σε μορφή Markdown.",
}



# Italian lexemes common.py
lexemes = {
    "module_llama_cpp_name": "Modulo llama.cpp",

    "module_llama_cpp_model_exception": "Errore modello non trovato: {error_msg}",
    "module_llama_cpp_task_exception": "Impossibile inizializzare il generatore di modelli: {error_msg}",
}



# Turkish lexemes toolbar.py
lexemes = {
    "actions_label_new_document": "Yeni belge",
    "actions_accessible_name_new_document": "Yeni belge",
    "actions_label_open": "Aç",
    "actions_accessible_name_open": "Belge aç",
    "actions_label_save": "Kaydet",
    "actions_accessible_name_save": "Belgeyi kaydet",
    "actions_label_save_as": "Farklı Kaydet",
    "actions_accessible_name_save_as": "Belgeyi farklı kaydet",
    "actions_label_sync": "Senkronize Et",
    "actions_accessible_name_sync": "Belgeyi dış depolama ile senkronize et",
    "actions_label_edit": "Düzenle",
    "actions_accessible_name_edit": "Dosyayı düzenle",
    "actions_label_source": "Kaynak",
    "actions_accessible_name_source": "Kaynak kodu",
    "actions_label_encrypt": "Şifrele",
    "actions_accessible_name_encrypt": "Dosyayı şifrele",
    "actions_label_decrypt": "Şifreyi Çöz",
    "actions_accessible_name_decrypt": "Dosyanın şifresini çöz",
    "actions_label_bold": "Kalın",
    "actions_accessible_name_bold": "Kalın metin formatı",
    "actions_label_italic": "İtalik",
    "actions_accessible_name_italic": "İtalik metin formatı",
    "actions_label_underline": "Altı Çizili",
    "actions_accessible_name_underline": "Altı çizili metin formatı",
    "actions_label_strikethrough": "Üstü Çizili",
    "actions_accessible_name_strikethrough": "Üstü çizili metin formatı",
    "actions_label_blockquote": "Alıntı",
    "actions_accessible_name_blockquote": "Alıntı metin formatı",
    "actions_label_color_picker": "Renk Seçici",
    "actions_accessible_name_color_picker": "Metin rengi seç",
    "actions_label_ai_assistant": "Yapay Zeka Asistanı",
    "actions_accessible_name_ai_assistant": "Yapay Zeka Asistanını başlat",
    "actions_label_about": "Hakkında",
    "actions_accessible_name_about": "Uygulama hakkında",
    "actions_label_exit": "Çıkış",
    "actions_accessible_name_exit": "Uygulamadan çık",
    "actions_label_settings": "Ayarlar",
    "actions_accessible_name_settings": "Uygulama ayarları",

    "search_input_label": "Arama",
    "search_input_placeholder_text": "Belgede metin ara",
    "search_input_accessible_description": "Arama giriş alanı",
    "search_buttons_label_clear": "Aramayı temizle",
    "search_buttons_accessible_name_clear": "Arama giriş alanını temizle",
    "search_buttons_label_prev": "Önceki arama",
    "search_buttons_accessible_name_prev": "Önceki arama sonucu",
    "search_buttons_label_next": "Sonraki arama",
    "search_buttons_accessible_name_next": "Sonraki arama sonucu",
    "search_case_sensitive_accessible_name": "Büyük/küçük harfe duyarlı",
    "search_case_sensitive_tooltip": "Büyük/küçük harfe duyarlı arama",
}



# French lexemes color_picker_dialog.py
lexemes = {
    "color_picker_title": "Sélectionner une couleur",

    "color_picker_color_black": "Noir",
    "color_picker_color_silver": "Argenté",
    "color_picker_color_gray": "Gris",
    "color_picker_color_white": "Blanc",
    "color_picker_color_maroon": "Marron",
    "color_picker_color_red": "Rouge",
    "color_picker_color_purple": "Violet",
    "color_picker_color_fuchsia": "Fuchsia",
    "color_picker_color_green": "Vert",
    "color_picker_color_lime": "Citron vert",
    "color_picker_color_olive": "Olive",
    "color_picker_color_yellow": "Jaune",
    "color_picker_color_navy": "Bleu marine",
    "color_picker_color_blue": "Bleu",
    "color_picker_color_teal": "Sarcelle",
    "color_picker_color_aqua": "Aqua",
    "color_picker_color_aliceblue": "Bleu Alice",
    "color_picker_color_antiquewhite": "Blanc antique",
    "color_picker_color_aquamarine": "Aigue-marine",
    "color_picker_color_azure": "Azur",
    "color_picker_color_beige": "Beige",
    "color_picker_color_bisque": "Bisque",
    "color_picker_color_blanchedalmond": "Amande blanchie",
    "color_picker_color_blueviolet": "Violet bleu",
    "color_picker_color_brown": "Brun",
    "color_picker_color_burlywood": "Bois robuste",
    "color_picker_color_cadetblue": "Bleu cadet",
    "color_picker_color_chartreuse": "Chartreuse",
    "color_picker_color_chocolate": "Chocolat",
    "color_picker_color_coral": "Corail",
    "color_picker_color_cornflowerblue": "Bleu bleuet",
    "color_picker_color_cornsilk": "Soie de maïs",
    "color_picker_color_crimson": "Cramoisi",
    "color_picker_color_cyan": "Cyan",
    "color_picker_color_darkblue": "Bleu foncé",
    "color_picker_color_darkcyan": "Cyan foncé",
    "color_picker_color_darkgoldenrod": "Verge d'or foncée",
    "color_picker_color_darkgray": "Gris foncé",
    "color_picker_color_darkgreen": "Vert foncé",
    "color_picker_color_darkkhaki": "Kaki foncé",
    "color_picker_color_darkmagenta": "Magenta foncé",
    "color_picker_color_darkolivegreen": "Vert olive foncé",
    "color_picker_color_darkorange": "Orange foncé",
    "color_picker_color_darkorchid": "Orchidée foncée",
    "color_picker_color_darkred": "Rouge foncé",
    "color_picker_color_darksalmon": "Saumon foncé",
    "color_picker_color_darkseagreen": "Vert mer foncé",
    "color_picker_color_darkslateblue": "Bleu ardoise foncé",
    "color_picker_color_darkslategray": "Gris ardoise foncé",
    "color_picker_color_darkturquoise": "Turquoise foncé",
    "color_picker_color_darkviolet": "Violet foncé",
    "color_picker_color_deeppink": "Rose profond",
    "color_picker_color_deepskyblue": "Bleu ciel profond",
    "color_picker_color_dimgray": "Gris terne",
    "color_picker_color_dodgerblue": "Bleu dodger",
    "color_picker_color_firebrick": "Brique de feu",
    "color_picker_color_floralwhite": "Blanc floral",
    "color_picker_color_forestgreen": "Vert forêt",
    "color_picker_color_gainsboro": "Gainsboro",
    "color_picker_color_ghostwhite": "Blanc fantôme",
    "color_picker_color_gold": "Or",
    "color_picker_color_goldenrod": "Verge d'or",
    "color_picker_color_greenyellow": "Vert jaune",
    "color_picker_color_honeydew": "Rosée de miel",
    "color_picker_color_hotpink": "Rose chaud",
    "color_picker_color_indianred": "Rouge indien",
    "color_picker_color_indigo": "Indigo",
    "color_picker_color_ivory": "Ivoire",
    "color_picker_color_khaki": "Kaki",
    "color_picker_color_lavender": "Lavande",
    "color_picker_color_lavenderblush": "Rouge lavande",
    "color_picker_color_lawngreen": "Vert gazon",
    "color_picker_color_lemonchiffon": "Mousseline de citron",
    "color_picker_color_lightblue": "Bleu clair",
    "color_picker_color_lightcoral": "Corail clair",
    "color_picker_color_lightcyan": "Cyan clair",
    "color_picker_color_lightgoldenrodyellow": "Jaune verge d'or clair",
    "color_picker_color_lightgray": "Gris clair",
    "color_picker_color_lightgreen": "Vert clair",
    "color_picker_color_lightpink": "Rose clair",
    "color_picker_color_lightsalmon": "Saumon clair",
    "color_picker_color_lightseagreen": "Vert mer clair",
    "color_picker_color_lightskyblue": "Bleu ciel clair",
    "color_picker_color_lightslategray": "Gris ardoise clair",
    "color_picker_color_lightsteelblue": "Bleu acier clair",
    "color_picker_color_lightyellow": "Jaune clair",
    "color_picker_color_limegreen": "Vert citron",
    "color_picker_color_linen": "Lin",
    "color_picker_color_magenta": "Magenta",
    "color_picker_color_mediumaquamarine": "Aigue-marine moyenne",
    "color_picker_color_mediumblue": "Bleu moyen",
    "color_picker_color_mediumorchid": "Orchidée moyenne",
    "color_picker_color_mediumpurple": "Violet moyen",
    "color_picker_color_mediumseagreen": "Vert mer moyen",
    "color_picker_color_mediumslateblue": "Bleu ardoise moyen",
    "color_picker_color_mediumspringgreen": "Vert printemps moyen",
    "color_picker_color_mediumturquoise": "Turquoise moyen",
    "color_picker_color_mediumvioletred": "Rouge violet moyen",
    "color_picker_color_midnightblue": "Bleu minuit",
    "color_picker_color_mintcream": "Crème de menthe",
    "color_picker_color_mistyrose": "Rose brumeux",
    "color_picker_color_moccasin": "Mocassin",
    "color_picker_color_navajowhite": "Blanc Navajo",
    "color_picker_color_oldlace": "Vieille dentelle",
    "color_picker_color_olivedrab": "Olive terne",
    "color_picker_color_orange": "Orange",
    "color_picker_color_orangered": "Rouge orange",
    "color_picker_color_orchid": "Orchidée",
    "color_picker_color_palegoldenrod": "Verge d'or pâle",
    "color_picker_color_palegreen": "Vert pâle",
    "color_picker_color_paleturquoise": "Turquoise pâle",
    "color_picker_color_palevioletred": "Rouge violet pâle",
    "color_picker_color_papayawhip": "Fouetté à la papaye",
    "color_picker_color_peachpuff": "Pêche",
    "color_picker_color_peru": "Pérou",
    "color_picker_color_pink": "Rose",
    "color_picker_color_plum": "Prune",
    "color_picker_color_powderblue": "Bleu poudre",
    "color_picker_color_rebeccapurple": "Pourpre Rebecca",
    "color_picker_color_rosybrown": "Brun rosé",
    "color_picker_color_royalblue": "Bleu royal",
    "color_picker_color_saddlebrown": "Brun selle",
    "color_picker_color_salmon": "Saumon",
    "color_picker_color_sandybrown": "Brun sable",
    "color_picker_color_seagreen": "Vert mer",
    "color_picker_color_seashell": "Coquillage",
    "color_picker_color_sienna": "Terre de Sienne",
    "color_picker_color_skyblue": "Bleu ciel",
    "color_picker_color_slateblue": "Bleu ardoise",
    "color_picker_color_slategray": "Gris ardoise",
    "color_picker_color_snow": "Neige",
    "color_picker_color_springgreen": "Vert printemps",
    "color_picker_color_steelblue": "Bleu acier",
    "color_picker_color_tan": "Tan",
    "color_picker_color_thistle": "Chardon",
    "color_picker_color_tomato": "Tomate",
    "color_picker_color_turquoise": "Turquoise",
    "color_picker_color_violet": "Violet",
    "color_picker_color_wheat": "Blé",
    "color_picker_color_whitesmoke": "Fumée blanche",
    "color_picker_color_yellowgreen": "Vert jaune",
}



# Turkish lexemes settings_dialog.py
lexemes = {
    "tab_openai_api_config": "OpenAI API",

    "module_openai_api_label": "OpenAI API",
    "module_openai_api_url_label": "API URL",
    "module_openai_api_url_input_placeholder_text": "API URL",
    "module_openai_api_url_input_accessible_description":
        "OpenAI API'sinin URL'si, API'nin uç noktasının adresidir ve hizmete veya sürüme bağlı olarak değişebilir.\n"
        "AI Asistanı, sohbet işlevselliği veya metin tamamlama için uç noktayı kullanır.\n"
        "Mevcut URL'yi almak için OpenAI API'nin resmi belgelerine başvurun.",
    "module_openai_api_key_label": "API Anahtarı",
    "module_openai_api_key_input_placeholder_text": "API Anahtarı",
    "module_openai_api_key_input_accessible_description":
        "OpenAI API Anahtarı, API uç noktasına yapılan istekleri doğrulamak için kullanılan gizli bir jetondur.",
    "module_openai_api_supported_models_label": "Desteklenen Modeller",
    "module_openai_api_model_names_combo_placeholder_text": "Bir Model Seçin",
    "module_openai_api_model_names_combo_accessible_description":
        "Sohbet konuşmaları için desteklenen modellerden seçim yapın.",

    "module_openai_api_base_system_prompt_label": "Sistem İstemleri",
    "module_openai_api_base_system_prompt_edit_placeholder_text": "Her istekten önce gelen temel sistem istemi",
    "module_openai_api_base_system_prompt_edit_accessible_description":
        "Her istekten önce gelen temel sistem istemi.\n"
        "Genellikle talimatlar veya rol özellikleri içeren düz metindir.",

    "module_openai_api_base_response_temperature_label": "Sıcaklık: {temperature}",
    "module_openai_api_base_response_temperature_input_accessible_description":
        "Modelin yanıtlarının rastgeleliğini ayarlar. Daha yüksek değerler daha çeşitli çıktılar üretir, "
        "daha düşük değerler ise yanıtları daha tahmin edilebilir yapar.",

    "module_openai_api_base_response_max_tokens_label": "Maksimum Yanıt Token'ları",
    "module_openai_api_base_response_max_tokens_input_accessible_description":
        "Yanıtta alınacak maksimum token sayısını belirler, bu da kelimeler ve noktalama işaretlerini içerir, "
        "çıktının uzunluğunu kontrol eder.",

    "module_openai_api_config_prompt_history_size_label": "İstem Geçmişi Boyutu",
    "module_openai_api_config_prompt_history_size_input_accessible_description":
        "Sistem tarafından referans için tutulan istem geçmişindeki giriş sayısını kontrol eder.\n"
        "Sıfır değer sınırsız girişe izin verir."
}



# Greek lexemes settings_dialog.py
lexemes = {
    "tab_ondevice_llm_config": "LLM στη Συσκευή",

    "module_ondevice_llm_config_label": "Μοντέλο LLM στη Συσκευή",
    "module_ondevice_llm_config_path_label": "Τοποθεσία Μοντέλου ONNX",
    "module_ondevice_llm_config_path_input_placeholder_text": "Διαδρομή προς τον κατάλογο του μοντέλου",
    "module_ondevice_llm_config_path_input_accessible_description":
        "Ένα πεδίο εισόδου με επιλογέα για να καθορίσετε την τοποθεσία του καταλόγου του μοντέλου όπου βρίσκονται\n"
        "τα αρχεία ONNX. Τα υποστηριζόμενα μοντέλα είναι σε μορφή ONNX, το οποίο αντιπροσωπεύει το Open Neural\n"
        "Network Exchange, ένα ανοιχτό πρότυπο για τα φορμάτ των μοντέλων μηχανικής μάθησης.",

    "module_ondevice_llm_config_response_temperature_label": "Θερμοκρασία: {temperature}",
    "module_ondevice_llm_config_response_temperature_input_accessible_description":
        "Ρυθμίζει την τυχαιότητα των απαντήσεων του μοντέλου. Υψηλότερες τιμές παράγουν πιο ποικίλες εξόδους,\n"
        "ενώ χαμηλότερες τιμές καθιστούν τις απαντήσεις πιο προβλέψιμες.",

    "module_ondevice_llm_config_response_max_tokens_label": "Μέγιστος Αριθμός Tokens Ανά Απάντηση",
    "module_ondevice_llm_config_response_max_tokens_input_accessible_description":
        "Ορίζει τον μέγιστο αριθμό των tokens που μπορούν να ληφθούν σε μία απάντηση, όπως λέξεις και σημεία στίξης,\n"
        "ελέγχοντας το μήκος της απόκρισης.",

    "module_ondevice_llm_config_prompt_history_size_label": "Μέγεθος Ιστορικού Προτροπών",
    "module_ondevice_llm_config_prompt_history_size_input_accessible_description":
        "Ελέγχει τον αριθμό των καταχωρήσεων στο ιστορικό προτροπών που διατηρεί το σύστημα για αναφορά.\n"
        "Μια τιμή μηδέν επιτρέπει απεριόριστες καταχωρήσεις."
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Part of the 'Module llama.cpp' module.
- Functionality: Provides helper functions to initialize and manage ONNX Runtime sessions for supported LLMs.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

import os
import logging
import asyncio

from threading import Lock

from typing import Union, Iterator

from llama_cpp import Llama, CreateChatCompletionResponse, CreateChatCompletionStreamResponse


class ModelHelper:

    model: Llama
    generator: Union[
        CreateChatCompletionResponse, Iterator[CreateChatCompletionStreamResponse]
    ]
    tokenizer: None  # LlamaTokenizer

    _instance = None  # Singleton instance
    _lock = Lock()

    search_options = {}

    def __new__(cls, *args, **kwargs):
        # Implement singleton pattern to ensure only one instance exists.
        if not cls._instance:
            with cls._lock:
                if not cls._instance:
                    # Create the instance if it doesn't exist
                    cls._instance = super().__new__(cls)
        return cls._instance

    @classmethod
    def reload(cls, *args, **kwargs):
        """
        Reinitialize the singleton instance to allow controlled re-creation.
        """
        with cls._lock:
            # Create a new instance
            cls._instance = super().__new__(cls, *args, **kwargs)

    def __init__(self, model_path: str, n_ctx: int = None, chat_format: str = None, search_options: dict = None):
        # Prevent reinitialization if the instance is already configured
        if hasattr(self, 'logger'):
            return

        # Use a lock to ensure initialization is thread-safe and atomic.
        with self._lock:
            # Double-check to prevent race conditions during initialization.
            if hasattr(self, 'logger'):
                return

            super(ModelHelper, self).__init__()

            self.logger = logging.getLogger('llama_cpp_model_helper')

            # Check model path is correct
            self.model_path = model_path
            if not os.path.isfile(self.model_path):
                self.logger.warning(f"Model not found in '{self.model_path}'")
                self.model_path = None

            # Context window. Refer to the logger for messages related to this value.
            # llama_new_context_with_model:
            #   n_ctx_per_seq (2048) < n_ctx_train (8192) -- the full capacity of the model will not be utilized
            self.n_ctx = n_ctx if n_ctx else 2048  # Default context window if not specified

            # Chat format specification
            self.chat_format = chat_format

            # Validate and set search options
            if search_options:
                supported_search_options = [
                    'temperature', 'top_p', 'top_k', 'min_p', 'typical_p', 'max_tokens',
                    'presence_penalty', 'frequency_penalty', 'repeat_penalty', 'tfs_z',
                    'mirostat_mode', 'mirostat_tau', 'mirostat_eta',  # Mirostat sampling
                ]
                # Check and filter supported
                search_options = {key: value for key, value in search_options.items() if key in supported_search_options}
                # Convert the integer value of temperature to a float
                if 'temperature' in search_options and isinstance(search_options['temperature'], int):
                    search_options['temperature'] = self.convert_temperature(search_options['temperature'])
                # Add or rewrite params if passed
                self.search_options.update(search_options)

    def init_model(self):
        """
        Lazily initializes the model with the given configuration. If the model is already initialized,
        the method exits early.

        Raises:
            ValueError: If `model_path` is not set or does not exist.
            RuntimeError: If an error occurs during the model instantiation due to configuration issues.
            Exception: For any unexpected errors.
        """

        # Check if the model is already initialized
        if getattr(self, 'model', None):
            return

        # Validate the model path
        if not self.model_path:
            raise ValueError(f"Model path is not set in '{type(self).__name__}'")

        self.logger.info(f"Initializing model: {self.model_path}")

        try:
            # Initialize the model
            self.model = Llama(
                model_path=self.model_path,
                chat_format=self.chat_format,  # e.g., 'chatml', 'llama-2', 'gemma', etc.
                n_ctx=self.n_ctx,  # Context window
                verbose=False  # Disable verbose output
            )
        except (RuntimeError, ValueError) as e:
            self.logger.error(f"Model initialization failed for path '{self.model_path}' with error: {e}")
            raise
        except Exception as e:
            self.logger.critical(f"Unexpected error during model initialization: {e}")
            raise

    def get_input_tokens(self, text):
        # Check if the model is initialized
        if not getattr(self, 'model', None):
            return None

        # Tokenize the input text using the model's tokenizer
        input_tokens = self.model.tokenize(text.encode("utf-8"))

        return input_tokens

    def init_generator(self, prompt_messages, search_options):
        """
        Initialize the completion stream with the given prompt and options.

        Output example:
        [
            {"role": "system", "content": "You are ..."},
            {
                "role": "user",
                "content": "Describe ..."
            }
        ]
        """

        # Check if the model is initialized
        if not getattr(self, 'model', None):
            return None

        generator = self.model.create_chat_completion(
            messages=prompt_messages,
            stream=True,
            **search_options
        )

        return generator

    async def async_wrap_iterator(self, iterator):
        # Asynchronously handle output from generator
        for chunk in iterator:
            delta = chunk['choices'][0]['delta']
            if 'role' in delta:
                output = f"{delta['role']}: "  # 'assistant: '
                self.logger.debug(f"Role output: {output}")
                await asyncio.sleep(0)  # Yield control to the event loop
                yield ''  # output
            elif 'content' in delta:
                tokens = delta['content']  # content might contain a space symbol, so do not do the split() on it
                output = f"{tokens}"
                self.logger.debug(f"Output token(s): {tokens}")
                await asyncio.sleep(0)  # Yield control to the event loop
                yield output
            elif 'finish_reason' in delta:
                # For instance, if max_tokens limit reached: 'finish_reason': 'length'
                if delta['finish_reason'] is not None:
                    self.logger.debug(f"Output finished with the reason: '{delta['finish_reason']}'")
                    yield ''

    async def generate_output(self, generator):
        # Generate outputs for the model asynchronously.
        async for item in self.async_wrap_iterator(generator):
            return item

        # To stop the iteration
        raise StopAsyncIteration

    def get_model_name(self):
        # Check if the model is initialized, verify it has a 'metadata' attribute,
        # and confirm that the 'general.name' key exists in metadata.
        if (getattr(self, 'model', None)
                and hasattr(self.model, 'metadata')
                and 'general.name' in self.model.metadata):
            return self.model.metadata['general.name']

        # Safely return the model name, truncating if too long.
        return ModelHelper.truncate_string(os.path.basename(self.model_path))

    @staticmethod
    def truncate_string(text, max_length=32):
        # Truncate the string to the specified length, adding an ellipsis if necessary.
        if len(text) <= max_length:
            return text

        # Calculating the space to be allocated to the start and end of the string around the ellipsis
        part_length = (max_length - 3) // 2
        start_part = text[:part_length]
        end_part = text[-part_length:]

        # Create the truncated string with an ellipsis in the middle
        return f"{start_part}...{end_part}"

    @staticmethod
    def convert_temperature(temperature: int = 0):
        """
        Convert an integer temperature to a float for use in model settings.
        """
        return temperature / 100



# Turkish lexemes common.py
lexemes = {
    "module_llama_cpp_name": "Modül llama.cpp",

    "module_llama_cpp_model_exception": "Model bulunamadı hatası: {error_msg}",
    "module_llama_cpp_task_exception": "Model jeneratörü başlatılamıyor: {error_msg}",
}



# Hindi lexemes settings_dialog.py
lexemes = {
    "window_title": "सेटिंग्स",

    "button_close": "बंद करें",

    "tab_general": "सामान्य",
    "tab_editor_config": "संपादक",
    "tab_viewer_config": "दर्शक",
    "tab_ai_config": "एआई कॉन्फ़िग",

    "general_app_config_label": "एप्लिकेशन कॉन्फ़िग",
    "general_app_language_label": "भाषा",
    "general_app_language_combo_placeholder_text": "एक भाषा चुनें",
    "general_app_language_combo_accessible_description": "एप्लिकेशन की इंटरफ़ेस भाषा",
    "general_app_theme_label": "थीम",
    "general_app_theme_combo_placeholder_text": "एक थीम चुनें",
    "general_app_theme_combo_accessible_description": "एप्लिकेशन की इंटरफ़ेस थीम",
    "general_app_main_menu_label": "मुख्य मेनू",
    "general_app_main_menu_checkbox": "मुख्य मेनू दिखाएं",
    "general_app_main_menu_checkbox_accessible_description": "ऐप का मुख्य ड्रॉपडाउन मेनू दिखाएं",
    "general_app_font_size_label": "फ़ॉन्ट आकार: {size} पीटी",
    "general_app_font_size_slider_accessible_description": "ऐप का वैश्विक फ़ॉन्ट आकार समायोजित करें",

    "general_statusbar_label": "स्थिति पट्टी",
    "general_statusbar_show_global_cursor_position_checkbox": "ग्लोबल कर्सर स्थिति दिखाएं",
    "general_statusbar_show_global_cursor_position_checkbox_accessible_description":
        "स्थिति पट्टी में ग्लोबल कर्सर स्थिति दिखाएं",

    "editor_config_label": "संपादक कॉन्फ़िगरेशन",
    "editor_config_show_line_numbers_checkbox": "लाइन संख्याएँ दिखाएं",
    "editor_config_show_line_numbers_checkbox_accessible_description": "संपादक में लाइन संख्याएँ दिखाएं",

    "viewer_config_label": "दर्शक कॉन्फ़िगरेशन",
    "viewer_config_process_emojis_checkbox": "पाठ इमोजी को ग्राफिक में परिवर्तित करें",
    "viewer_config_process_emojis_checkbox_accessible_description": "पाठ इमोजी को चित्रित प्रतिनिधि में परिवर्तित करें",
    "viewer_config_highlight_todos_checkbox": "TODO को हाइलाइट करें",
    "viewer_config_highlight_todos_checkbox_accessible_description": "पाठ में TODO टैग को महत्वपूर्ण बनाएं",
    "viewer_config_open_link_confirmation_checkbox": "लिंक खोलने के लिए पुष्टि की आवश्यकता",
    "viewer_config_open_link_confirmation_checkbox_accessible_description":
        "लिंक खोलने से पहले पुष्टि की आवश्यकता है",
    "viewer_config_save_resources_checkbox": "बाहरी छवियों को डिस्क में स्वचालित रूप से सहेजें",
    "viewer_config_save_resources_checkbox_accessible_description":
        "ऑफ़लाइन पहुंच के लिए बाहरी छवियों की स्वचालित प्रतिलिपियाँ डिस्क में सहेजें।",

    "ai_config_inference_module_label": "अनुमान मॉड्यूल",
    "ai_config_inference_module_names_combo_label": "सक्रिय अनुमान मॉड्यूल",
    "ai_config_inference_module_names_combo_placeholder_text": "मॉड्यूल चुनें",
    "ai_config_inference_module_names_combo_accessible_description":
        "AI सहायक के साथ संचालन के लिए उपलब्ध AI अनुमान मॉड्यूलों में से चुनें।\n"
        "विकल्पों में वास्तविक समय प्रसंस्करण के साथ बड़े भाषा मॉडल (LLM) या API-आधारित कार्यक्षमताएं शामिल हैं।",

    "ai_config_base_label": "मूल पैरामीटर",
    "ai_config_multi_turn_dialogue_checkbox": "संवादी स्मृति के साथ मल्टी-टर्न संवाद",
    "ai_config_multi_turn_dialogue_checkbox_accessible_description":
        "संवादी स्मृति के लिए अंतिम संकेत को बनाए रखने वाला मल्टी-टर्न संवाद सक्षम करें।\n"
        "बंद होने पर, केवल नया संदेश और सिस्टम प्रॉम्प्ट प्रतिक्रिया को प्रभावित करते हैं।",
    "ai_config_convert_to_md_checkbox": "परिणाम को Markdown में बदलें",
    "ai_config_convert_to_md_checkbox_accessible_description":
        "आउटपुट संदेश को Markdown प्रारूप में बदलें।",
}



# German lexemes settings_dialog.py
lexemes = {
    "tab_openai_api_config": "OpenAI API",

    "module_openai_api_label": "OpenAI API",
    "module_openai_api_url_label": "API-URL",
    "module_openai_api_url_input_placeholder_text": "API-URL",
    "module_openai_api_url_input_accessible_description":
        "Die URL der OpenAI-API ist die Adresse des API-Endpunkts, die je nach Dienst und Version variieren kann.\n"
        "Der KI-Assistent verwendet diejenige für die Funktionalität von Chatgesprächen oder Textvervollständigungen.\n"
        "Bitte konsultieren Sie die offizielle Dokumentation der OpenAI-API, um die aktuelle URL zu erhalten.",
    "module_openai_api_key_label": "API-Schlüssel",
    "module_openai_api_key_input_placeholder_text": "API-Schlüssel",
    "module_openai_api_key_input_accessible_description":
        "Der OpenAI-API-Schlüssel ist ein geheimer Token, der zur Authentifizierung\n"
        "von Anfragen an den API-Endpunkt verwendet wird.",
    "module_openai_api_supported_models_label": "Unterstützte Modelle",
    "module_openai_api_model_names_combo_placeholder_text": "Modell wählen",
    "module_openai_api_model_names_combo_accessible_description":
        "Wählen Sie aus unterstützten Modellen für Chat-Gespräche aus.",

    "module_openai_api_base_system_prompt_label": "Systemprompt",
    "module_openai_api_base_system_prompt_edit_placeholder_text": "Basis-Systemprompt, der jeder Anfrage vorausgeht",
    "module_openai_api_base_system_prompt_edit_accessible_description":
        "Ein Basis-Systemprompt, der jeder Anfrage vorausgeht.\n"
        "In der Regel handelt es sich um einfachen Text mit Anweisungen oder Rollenmerkmalen.",

    "module_openai_api_base_response_temperature_label": "Temperatur: {temperature}",
    "module_openai_api_base_response_temperature_input_accessible_description":
        "Passt die Zufälligkeit der Modellantworten an. Höhere Werte produzieren vielfältigere Ausgaben, "
        "während niedrigere Werte die Antworten vorhersehbarer machen.",

    "module_openai_api_base_response_max_tokens_label": "Maximale Antwort-Token",
    "module_openai_api_base_response_max_tokens_input_accessible_description":
        "Maximale Anzahl von Token, die in einer Antwort empfangen werden, wie Wörter und Zeichensetzung, "
        "steuert die Länge der Ausgabe.",

    "module_openai_api_config_prompt_history_size_label": "Größe der Prompt-Historie",
    "module_openai_api_config_prompt_history_size_input_accessible_description":
        "Steuert die Anzahl der Einträge in der Prompt-Historie, die das System zur Referenz behält.\n"
        "Ein Wert von null ermöglicht unbegrenzte Einträge."
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Text view widget.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import Signal
from PySide6.QtGui import QFont
from PySide6.QtWidgets import QTextBrowser

from .app_config import AppConfig

import logging


class ViewWidget(QTextBrowser):
    """
    Class to implement document view functionality.
    It utilises QTextBrowser power by extending or modifying of the original widget methods.
    """

    content_set = Signal()

    def __init__(self, parent=None):
        """
        Args:
            parent (optional): Parent object
        """
        super(ViewWidget, self).__init__(parent)

        # Get app's global font size
        font = QFont()
        font.setPointSize(AppConfig().get_font_size())
        self.setFont(font)

        self.logger = logging.getLogger('view_widget')

        # Initialize storage for positions and indexes of searched text occurrences.
        self._searched_text_positions = []

    def setDocument(self, document):
        # Override setDocument() to allow additional actions like emit the document set signal
        super().setDocument(document)
        # Emit document set signal to notify listeners
        self.content_set.emit()

    def setHtml(self, html_text):
        # Override setDocument() to allow additional actions like emit the document set signal
        super().setHtml(html_text)
        # Emit document set signal to notify listeners
        self.content_set.emit()

    def mousePressEvent(self, event):
        super().mousePressEvent(event)

        cursor_pos = event.pos()

        anchor = self.anchorAt(cursor_pos)
        # Ensure hyperlink clicks are not processed when clicking on expandable/collapsible blocks.
        if anchor and (anchor.startswith('expandable') or anchor.startswith('collapsible')):
            self.logger.debug("Anchor URL at cursor:", anchor)
            return

    def contextMenuEvent(self, event):
        # Create the standard context menu
        menu = self.createStandardContextMenu()

        # Modify the menu font to hint the size
        menu.setFont(self.font())

        """
        # Optionally add new actions or modify existing ones
        customAction = menu.addAction("Custom Action")
        customAction.triggered.connect(self.custom_action_triggered)
        """

        # Display the menu
        menu.exec_(event.globalPos())

    def loadResource(self, resource_type, url):
        # Log the resource request
        self.logger.debug(f"Requesting resource of type {resource_type} at URL: {url.toString()}")

    def searched_text_count(self, searched_text, find_flags) -> int:
        """
        Counts the number of occurrences of a given text within the document.

        Args:
            searched_text (str): The text string to search for within the document.
            find_flags (QTextDocument.FindFlags): Flags to control the search behavior,
                                                  such as case sensitivity or whole word matching.

        Returns:
            int: The number of times the searched text appears in the document.
        """
        count = 0
        # Reset stored text positions from previous searches to prepare for a new search.
        self._searched_text_positions = []
        # Create a temporary cursor for the search
        # This way, the actual cursor position that the user sees will not be affected.
        temp_cursor = self.textCursor()

        # Move the temporary cursor to the start of the document
        temp_cursor.movePosition(temp_cursor.MoveOperation.Start)

        # Find and count occurrences
        while temp_cursor := self.document().find(searched_text, temp_cursor, find_flags):
            count += 1
            pos = temp_cursor.position()
            if pos not in self._searched_text_positions:
                self._searched_text_positions.append(pos)

        # Reset the stored positions if no occurrence index is found.
        if count == 0:
            self._searched_text_positions = []

        # No need to reset the visual cursor since the original cursor was not moved
        return count

    def searched_text_index(self, position) -> int:
        """
        Shows the index of the occurrence at a given cursor position within the document.

        Args:
            position (int): The cursor position within the document.

        Returns:
            int: The index of the occurrence at a given cursor position within the document.
        """

        if position in self._searched_text_positions:
            return self._searched_text_positions.index(position) + 1
        else:
            return 0



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

import pytest

import emoji
import markdown


@pytest.fixture
def markdown_obj():
    # Fixture to create and return main Markdown instance
    extensions = ['markdown.extensions.extra']
    # Init markdown object with the selected extensions
    md = markdown.Markdown(extensions=extensions)
    yield md


def test_markdown_conversion(markdown_obj):

    md_content = "*Italic text*"
    "**Bold text**"
    "___Italic bold text___"

    # Convert markdown to HTML
    html_content = markdown_obj.convert(md_content)

    assert html_content == "<p><em>Italic text</em></p>"
    "<p><strong>Bold text</strong></p>"
    "<p><strong><em>Bold text</em></strong></p>"


def test_emoji_conversion():

    text_content = ":cat:"

    # Process emojis :cat: to 🐱 conversion
    emoji_content = emoji.emojize(text_content, language="en")

    assert emoji_content == "🐈"



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtWidgets import QApplication

from notolog.app_config import AppConfig

import os
import sys
import pytest

os.environ.pop("QT_QPA_PLATFORM", None)
# Force Qt style override to avoid:
# QApplication: invalid style override 'kvantum' passed, ignoring it.
#    Available styles: Windows, Fusion
os.environ["QT_STYLE_OVERRIDE"] = "Fusion"


@pytest.fixture
def test_app(mocker):
    # Main widgets application; new
    # More info: https://doc.qt.io/qt-6/qcoreapplication.html#details
    app = QApplication.instance()
    if not app:
        # Consider: Fixture to initialize QCoreApplication for Non-GUI Tests
        """
        Virtual Framebuffer on Linux (Xvfb):
        For Linux, use Xvfb to run tests requiring GUI in a virtual framebuffer.
        For Windows, ensure GUI sessions are available for tests.
        """
        app = QApplication(sys.argv)

        # To correctly set up app settings
        app.setOrganizationName(AppConfig().get_settings_org_name())
        app.setApplicationName(AppConfig().get_settings_app_name_qa())

        AppConfig().set_test_mode(True)

    yield app

    # Cleanup
    app.quit()
    app.processEvents()



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Settings helper class.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from cryptography.fernet import Fernet
from cryptography.fernet import InvalidToken, InvalidSignature

from . import AppConfig

from typing import Any, Union

import os
import logging


class SettingsHelper:
    """
    Settings helper to perform additional operations like encrypting/decrypting sensitive data with
    symmetric encryption algorithm to store it in settings, as settings file may be exposed.
    """

    env_secret_name = "NOTOLOG_APP_SECRET_KEY"

    def __init__(self):
        super().__init__()

        self.logger = logging.getLogger('settings_helper')

        self.logger.info('Settings helper is activated')

        self.key = self.get_app_key()

    def encrypt_data(self, data) -> Union[str, None]:
        if not data:
            return data  # Empty default
        data_encoded = data.encode('utf-8')
        try:
            # Encrypt data using Fernet symmetric encryption.
            cipher_suite = Fernet(self.key)
            encrypted_data = cipher_suite.encrypt(data_encoded)
            return encrypted_data.decode('utf-8')
        except (InvalidToken, InvalidSignature, TypeError) as e:
            self.logger.warning(f'Settings helper encryption token error: {e}')
        except Exception as e:
            self.logger.warning(f'An unexpected error occurred while encrypting data: {e}')
        return None

    def decrypt_data(self, encrypted_data: Any) -> Union[str, None]:
        if not encrypted_data:
            # When settings not set
            return encrypted_data  # Empty default
        if not isinstance(encrypted_data, bytes):
            encrypted_data = encrypted_data.encode('utf-8')
        try:
            # Decrypt data using Fernet symmetric encryption.
            cipher_suite = Fernet(self.key)
            decrypted_data = cipher_suite.decrypt(encrypted_data)
            return decrypted_data.decode('utf-8')
        except (InvalidToken, InvalidSignature, TypeError) as e:
            self.logger.warning(f'Settings helper encryption token error: {e}')
        except Exception as e:
            self.logger.warning(f'An unexpected error occurred while decrypting data: {e}')
        return None

    def get_app_key(self) -> bytes:
        # Get the key from the system env first
        app_key_str = os.getenv(self.env_secret_name)

        # Trying to access key via the app config
        if app_key_str is None:
            app_key_str = AppConfig().get_security_app_secret()
            if app_key_str:
                # Store key with system envs
                os.environ[self.env_secret_name] = app_key_str

        # If no key found create a new one
        if not app_key_str:
            self.logger.info("Creating app new secret key")
            # Generate app new key. This is a base64-encoded bytes string
            app_key = self.generate_app_key()
            # Set up app new key
            self.setup_app_key(app_key)
            # Return new 32 url-safe base64-encoded bytes key
            return app_key
        else:
            # Convert the retrieved string back to bytes
            return app_key_str.encode('ascii')

    def setup_app_key(self, app_key: bytes) -> None:
        # Convert bytes to a string (decoding by ASCII)
        app_key_str = app_key.decode('ascii')
        # Store key in system envs
        os.environ[self.env_secret_name] = app_key_str
        # Store key in app config
        AppConfig().set_security_app_secret(app_key_str)

    @staticmethod
    def generate_app_key() -> bytes:
        # Generate a key without password, for app needs.
        return Fernet.generate_key()

    def _recreate_app_key(self):
        """
        Proof-of-Concept method; Avoid to use it in scenarios different from data repairing.
        """
        _new_app_key = self.generate_app_key()
        self.setup_app_key(_new_app_key)
        self.key = _new_app_key
        # Update settings value(s) upon that
        # value = self.encrypt_data(value)
        # settings.setValue(param_name, value)

    def parse_object_name(self, object_name: str) -> list:
        if object_name.__contains__(":"):
            object_name_parts = object_name.split(":")
            if len(object_name_parts) == 2:
                # lexeme_key, setting_name
                return object_name_parts
            else:
                self.logger.warning(f"Object name in a wrong format '{object_name}'")
        else:
            # lexeme_key, setting_name
            return [object_name, object_name]



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Main file to start up the app and set up an async loop.
- Functionality: This file is the main entry point for the Notolog app. It handles the initialization of the
  application environment, processes command line arguments, and starts the main application loop.

Detailed Description:
- Initializes the GUI by calling the main module.
- Sets up global logging and debug settings.
- Initializes the async loop.

Usage:
Run the module directly from the command line with the necessary arguments:
`python -m notolog.app`

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""


import argparse
import logging
import asyncio
import sys
import os

from PySide6.QtCore import QLoggingCategory
from PySide6.QtWidgets import QStyleFactory

from notolog.app_config import AppConfig
from notolog.font_loader import FontLoader

# Force Qt API (for qasync).
# It's necessary to set the QT_API environment variable before importing qasync
# because the library uses this environment variable to determine which Qt binding to use.
os.environ["QT_API"] = "PySide6"

from notolog.notolog_editor import NotologEditor  # noqa

from qasync import QEventLoop, QApplication  # noqa

# Force Qt style override
os.environ["QT_STYLE_OVERRIDE"] = "Fusion"


def main():
    # Check if any command line arguments are present
    if len(sys.argv) > 1:
        class NotologArgumentParser(argparse.ArgumentParser):
            def print_help(self, file=None):
                # Default command line intro
                print("\033[90m", end='')
                print("░░░░░░░░░░░░░░░░░░░▒▒▒▒▒▒▒▒▒▒▒▒▒▓▓▓▓▓▓▓███"
                      "\033[97m NOTOLOG \033[90m"
                      "███▓▓▓▓▓▓▓▒▒▒▒▒▒▒▒▒▒▒▒▒░░░░░░░░░░░░░░░░░░░")
                print("\033[92m", end='')
                print("╔═══════════════════════════════════════════════════════════════════════════════════════════╗")
                print("║ Notolog Editor is a GUI application. Simply run the command without any options to start. ║")
                print("╚═══════════════════════════════════════════════════════════════════════════════════════════╝")
                print("\033[0m")
                # Call the super method to print the standard help message
                super().print_help()

        # Create the parser
        parser = NotologArgumentParser(description="Notolog Editor: An open-source Markdown editor built with Python.")

        # Add a version argument
        parser.add_argument('-v', '--version', action='version',
                            version=f'{AppConfig().get_app_name()} {AppConfig().get_app_version()}',
                            help='show the version information and exit')

        # Parse the arguments
        # parser.parse_args()
        args = parser.parse_args()

        # Check if the --init=package option was provided
        if args.version:
            pass

        sys.exit(0)

    """
    Possible params:
    filename='notolog.log'
    format='%(asctime)s:%(levelname)s:%(name)s:%(message)s'
    """
    logger_level = AppConfig().get_logger_level()
    logging.basicConfig(level=logger_level, format='[%(name)s] %(funcName)s: %(levelname)s: %(message)s')

    logger = logging.getLogger('notolog')

    logger.info("%s v%s" % (AppConfig().get_app_name(), AppConfig().get_app_version()))
    logger.info("%s" % (AppConfig().get_app_license()))

    if logger_level > logging.DEBUG:
        # Suppress font-related messages in the logs
        QLoggingCategory.setFilterRules("qt.qpa.fonts=false")

    # Main application
    app = QApplication(sys.argv)
    # To correctly set up app settings
    app.setOrganizationName(AppConfig().get_settings_org_name())
    app.setOrganizationDomain(AppConfig().get_settings_org_domain())
    # Consider different app names for pip package and for the source files run,
    # as the settings storage depends on it.
    app.setApplicationName(AppConfig().get_settings_app_name())
    app.setApplicationVersion(AppConfig().get_app_version())

    # Custom styling may not render as expected with these themes.
    """
    # Detect the operating system to choose the style
    current_os = platform.system()
    if current_os == "Windows":
        app.setStyle(QStyleFactory.create("WindowsVista"))
    elif current_os == "Darwin":  # macOS
        app.setStyle(QStyleFactory.create("Macintosh"))
    else:  # Or: current_os == "Linux"
    """
    app.setStyle(QStyleFactory.create("Fusion"))  # Fusion is a cross-platform choice

    # Maintain a unique style regardless of the user's system settings
    app.setDesktopSettingsAware(False)

    # E.g. /usr/bin and /usr/bin/python3.11
    logger.debug(f'Application dir path "{app.applicationDirPath()}"; file path "{app.applicationFilePath()}"')

    # Init the application fonts
    FontLoader.init_fonts(app)

    # Get the screen to pass it to the main module
    screen = app.screens()[0]

    # Main loop
    loop = QEventLoop(app)
    asyncio.set_event_loop(loop)

    app_close_event = asyncio.Event()
    app.aboutToQuit.connect(app_close_event.set)

    # Start up the editor
    editor = NotologEditor(screen=screen)
    editor.show()

    with loop:
        """
        Contains run_forever() which is contain app.exec(),
        No need to run sys.exit(app.exec()) in this case.
        """
        loop.run_until_complete(app_close_event.wait())


if __name__ == '__main__':
    # Debug:
    # async def main(): ...
    # asyncio.run(main(), debug=True)
    main()



# Italian lexemes color_picker_dialog.py
lexemes = {
    "color_picker_title": "Seleziona un colore",

    "color_picker_color_black": "Nero",
    "color_picker_color_silver": "Argento",
    "color_picker_color_gray": "Grigio",
    "color_picker_color_white": "Bianco",
    "color_picker_color_maroon": "Marrone",
    "color_picker_color_red": "Rosso",
    "color_picker_color_purple": "Porpora",
    "color_picker_color_fuchsia": "Fucsia",
    "color_picker_color_green": "Verde",
    "color_picker_color_lime": "Lime",
    "color_picker_color_olive": "Oliva",
    "color_picker_color_yellow": "Giallo",
    "color_picker_color_navy": "Blu marino",
    "color_picker_color_blue": "Blu",
    "color_picker_color_teal": "Verde acqua",
    "color_picker_color_aqua": "Acqua",
    "color_picker_color_aliceblue": "Blu Alice",
    "color_picker_color_antiquewhite": "Bianco antico",
    "color_picker_color_aquamarine": "Acquamarina",
    "color_picker_color_azure": "Azzurro",
    "color_picker_color_beige": "Beige",
    "color_picker_color_bisque": "Biscotto",
    "color_picker_color_blanchedalmond": "Mandorla sbiancata",
    "color_picker_color_blueviolet": "Blu violaceo",
    "color_picker_color_brown": "Marrone",
    "color_picker_color_burlywood": "Legno massiccio",
    "color_picker_color_cadetblue": "Blu cadetto",
    "color_picker_color_chartreuse": "Chartreuse",
    "color_picker_color_chocolate": "Cioccolato",
    "color_picker_color_coral": "Corallo",
    "color_picker_color_cornflowerblue": "Blu fiordaliso",
    "color_picker_color_cornsilk": "Seta di mais",
    "color_picker_color_crimson": "Cremisi",
    "color_picker_color_cyan": "Ciano",
    "color_picker_color_darkblue": "Blu scuro",
    "color_picker_color_darkcyan": "Ciano scuro",
    "color_picker_color_darkgoldenrod": "Oro scuro",
    "color_picker_color_darkgray": "Grigio scuro",
    "color_picker_color_darkgreen": "Verde scuro",
    "color_picker_color_darkkhaki": "Khaki scuro",
    "color_picker_color_darkmagenta": "Magenta scuro",
    "color_picker_color_darkolivegreen": "Verde oliva scuro",
    "color_picker_color_darkorange": "Arancione scuro",
    "color_picker_color_darkorchid": "Orchidea scura",
    "color_picker_color_darkred": "Rosso scuro",
    "color_picker_color_darksalmon": "Salmone scuro",
    "color_picker_color_darkseagreen": "Verde mare scuro",
    "color_picker_color_darkslateblue": "Blu ardesia scuro",
    "color_picker_color_darkslategray": "Grigio ardesia scuro",
    "color_picker_color_darkturquoise": "Turchese scuro",
    "color_picker_color_darkviolet": "Viola scuro",
    "color_picker_color_deeppink": "Rosa profondo",
    "color_picker_color_deepskyblue": "Blu cielo profondo",
    "color_picker_color_dimgray": "Grigio tenue",
    "color_picker_color_dodgerblue": "Blu dodger",
    "color_picker_color_firebrick": "Mattone",
    "color_picker_color_floralwhite": "Bianco floreale",
    "color_picker_color_forestgreen": "Verde foresta",
    "color_picker_color_gainsboro": "Gainsboro",
    "color_picker_color_ghostwhite": "Bianco fantasma",
    "color_picker_color_gold": "Oro",
    "color_picker_color_goldenrod": "Oro vecchio",
    "color_picker_color_greenyellow": "Verde giallo",
    "color_picker_color_honeydew": "Melata",
    "color_picker_color_hotpink": "Rosa acceso",
    "color_picker_color_indianred": "Rosso indiano",
    "color_picker_color_indigo": "Indaco",
    "color_picker_color_ivory": "Avorio",
    "color_picker_color_khaki": "Khaki",
    "color_picker_color_lavender": "Lavanda",
    "color_picker_color_lavenderblush": "Lavanda chiara",
    "color_picker_color_lawngreen": "Verde prato",
    "color_picker_color_lemonchiffon": "Chiffon limone",
    "color_picker_color_lightblue": "Blu chiaro",
    "color_picker_color_lightcoral": "Corallo chiaro",
    "color_picker_color_lightcyan": "Ciano chiaro",
    "color_picker_color_lightgoldenrodyellow": "Giallo oro chiaro",
    "color_picker_color_lightgray": "Grigio chiaro",
    "color_picker_color_lightgreen": "Verde chiaro",
    "color_picker_color_lightpink": "Rosa chiaro",
    "color_picker_color_lightsalmon": "Salmone chiaro",
    "color_picker_color_lightseagreen": "Verde mare chiaro",
    "color_picker_color_lightskyblue": "Blu cielo chiaro",
    "color_picker_color_lightslategray": "Grigio ardesia chiaro",
    "color_picker_color_lightsteelblue": "Blu acciaio chiaro",
    "color_picker_color_lightyellow": "Giallo chiaro",
    "color_picker_color_limegreen": "Verde lime",
    "color_picker_color_linen": "Lino",
    "color_picker_color_magenta": "Magenta",
    "color_picker_color_mediumaquamarine": "Acquamarina medio",
    "color_picker_color_mediumblue": "Blu medio",
    "color_picker_color_mediumorchid": "Orchidea media",
    "color_picker_color_mediumpurple": "Porpora medio",
    "color_picker_color_mediumseagreen": "Verde mare medio",
    "color_picker_color_mediumslateblue": "Blu ardesia medio",
    "color_picker_color_mediumspringgreen": "Verde primavera medio",
    "color_picker_color_mediumturquoise": "Turchese medio",
    "color_picker_color_mediumvioletred": "Rosso violetto medio",
    "color_picker_color_midnightblue": "Blu mezzanotte",
    "color_picker_color_mintcream": "Crema di menta",
    "color_picker_color_mistyrose": "Rosa nebbia",
    "color_picker_color_moccasin": "Mocassino",
    "color_picker_color_navajowhite": "Bianco Navajo",
    "color_picker_color_oldlace": "Pizzo antico",
    "color_picker_color_olivedrab": "Oliva opaco",
    "color_picker_color_orange": "Arancione",
    "color_picker_color_orangered": "Rosso arancio",
    "color_picker_color_orchid": "Orchidea",
    "color_picker_color_palegoldenrod": "Oro pallido",
    "color_picker_color_palegreen": "Verde pallido",
    "color_picker_color_paleturquoise": "Turchese pallido",
    "color_picker_color_palevioletred": "Rosso violetto pallido",
    "color_picker_color_papayawhip": "Papaya",
    "color_picker_color_peachpuff": "Pesca",
    "color_picker_color_peru": "Perù",
    "color_picker_color_pink": "Rosa",
    "color_picker_color_plum": "Prugna",
    "color_picker_color_powderblue": "Blu polvere",
    "color_picker_color_rebeccapurple": "Porpora Rebecca",
    "color_picker_color_rosybrown": "Marrone rosato",
    "color_picker_color_royalblue": "Blu reale",
    "color_picker_color_saddlebrown": "Marrone sella",
    "color_picker_color_salmon": "Salmone",
    "color_picker_color_sandybrown": "Marrone sabbia",
    "color_picker_color_seagreen": "Verde mare",
    "color_picker_color_seashell": "Conchiglia",
    "color_picker_color_sienna": "Terra di Siena",
    "color_picker_color_skyblue": "Blu cielo",
    "color_picker_color_slateblue": "Blu ardesia",
    "color_picker_color_slategray": "Grigio ardesia",
    "color_picker_color_snow": "Neve",
    "color_picker_color_springgreen": "Verde primavera",
    "color_picker_color_steelblue": "Blu acciaio",
    "color_picker_color_tan": "Cuoio",
    "color_picker_color_thistle": "Cardo",
    "color_picker_color_tomato": "Pomodoro",
    "color_picker_color_turquoise": "Turchese",
    "color_picker_color_violet": "Viola",
    "color_picker_color_wheat": "Grano",
    "color_picker_color_whitesmoke": "Fumo bianco",
    "color_picker_color_yellowgreen": "Verde giallastro",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Extends QSortFilterProxyModel.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import QModelIndex, QSortFilterProxyModel, QRegularExpression

import os
import logging

from ..helpers.file_helper import remove_trailing_numbers


class SortFilterProxyModel(QSortFilterProxyModel):

    def __init__(self, extensions: list = None, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.logger = logging.getLogger('sort_filter_proxy_model')

        self._extensions = extensions  # Allow specific file extensions

    def get_extensions(self):
        # Get allowed file extensions
        return self._extensions

    def set_extensions(self, extensions):
        # Set allowed file extensions
        self._extensions = extensions
        self.invalidateFilter()  # Reapply the filter to the model to update view

    def add_extension(self, extension):
        # Add specific file extension
        if extension not in self._extensions:
            self._extensions.append(extension)
            self.invalidateFilter()  # Reapply the filter with the new extension

    def remove_extension(self, extension):
        # Remove specific file extension
        if extension in self._extensions:
            self._extensions.remove(extension)
            self.invalidateFilter()  # Reapply the filter without the removed extension

    def filterAcceptsRow(self, source_row: int, source_parent: QModelIndex) -> bool:
        source_model = self.sourceModel()
        index = source_model.index(source_row, 0, source_parent)
        if not index.isValid():
            return False

        regex = self.filterRegularExpression()  # Retrieve regex set with setFilterRegularExpression()
        if regex and regex.pattern():
            re = QRegularExpression(regex)
            match = re.match(index.data())
            if not (match.capturedTexts() and match.captured()):
                self.logger.debug('Filter row: %s' % index.data())
                return False

        if source_model.isDir(index):  # noqa
            return True  # Always show directories
        else:
            file_path = source_model.filePath(index)  # noqa
            extension = file_path.split(".")[-1]
            return not self._extensions or remove_trailing_numbers(extension).lower() in self._extensions

    def lessThan(self, left: QModelIndex, right: QModelIndex) -> bool:
        # Get the source model
        source_model = self.sourceModel()

        # Get the file names for the left and the right indexes
        left_name = source_model.fileName(left)  # noqa
        right_name = source_model.fileName(right)  # noqa

        # Sort "dotdot" always at the top (check the "dot" is not visible)
        if left_name == ".." and right_name != "..":
            return True
        elif right_name != ".." and left_name == "..":
            return False

        left_path = source_model.filePath(left)  # noqa
        right_path = source_model.filePath(right)  # noqa

        if os.path.isdir(left_path) and not os.path.isdir(right_path):
            return True
        elif not os.path.isdir(left_path) and os.path.isdir(right_path):
            return False

        # Default sorting order
        return super().lessThan(left, right)



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Provides a file renaming dialog UI.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import QSize
from PySide6.QtWidgets import QInputDialog, QLineEdit, QDialogButtonBox, QSizePolicy

from . import Settings
from . import Lexemes

import logging


class RenameFileDialog(QInputDialog):

    def __init__(self, file_name: str, parent=None):
        super().__init__(parent)

        self.file_name = file_name
        self.parent = parent

        if self.parent and hasattr(self.parent, 'font'):
            # Apply font from the main window to the dialog
            self.setFont(self.parent.font())

        self.settings = Settings(parent=self)

        self.logger = logging.getLogger('rename_file_dialog')

        # Load lexemes for selected language and scope
        self.lexemes = Lexemes(self.settings.app_language, default_scope='common')

        self.init_ui()

    def init_ui(self):
        """
        More info about the enum: https://doc.qt.io/qt-6/qinputdialog.html#InputDialogOption-enum
        dialog.setOption(QInputDialog.InputDialogOption.UsePlainTextEditForTextInput)
        """
        self.setInputMode(QInputDialog.InputMode.TextInput)
        self.setTextEchoMode(QLineEdit.EchoMode.Normal)
        self.setWindowTitle(self.lexemes.get('dialog_file_rename_title'))
        self.setLabelText(self.lexemes.get('dialog_file_rename_field_label'))
        self.setOkButtonText(self.lexemes.get('dialog_file_rename_button_ok'))
        self.setTextValue(self.file_name)

        self.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)
        if self.sizeHint().isValid():
            self.setMinimumSize(self.sizeHint())

        # Set dialog size derived from the main window size
        main_window_size = self.parent.size()
        dialog_width = int(main_window_size.width() * 0.33)
        dialog_height = int(main_window_size.height() * 0.33)
        self.resize(dialog_width, dialog_height)

        """
        Or use fixed size dialog:
        dialog.setFixedSize(dialog_width, dialog_height)
        """
        self.setMinimumSize(QSize(dialog_width, dialog_height))
        # Get the button box and adjust its alignment to center
        button_box = self.findChild(QDialogButtonBox)

        if isinstance(button_box, QDialogButtonBox):
            button_box.setCenterButtons(True)

        """
        Example with color palette:
        pal = dialog.palette()
        pal.setColor(dialog.backgroundRole(), QColor("cyan"))
        dialog.setPalette(pal)
        """



# Spanish lexemes settings_dialog.py
lexemes = {
    "tab_module_llama_cpp_config": "Módulo llama.cpp",

    "module_llama_cpp_config_label": "Módulo llama.cpp",
    "module_llama_cpp_config_path_label": "Ubicación del modelo",
    "module_llama_cpp_config_path_input_placeholder_text": "Seleccione o introduzca la ruta del modelo",
    "module_llama_cpp_config_path_input_accessible_description":
        "Un campo de entrada con un selector para especificar la ruta del modelo local. Admite modelos en formato GGUF,\n"
        "un formato de archivo binario optimizado para almacenar modelos utilizados con GGML y ejecutores basados en GGML.",
    "module_llama_cpp_config_path_input_filter_text": "Archivos GGUF",

    "module_llama_cpp_config_context_window_label": "Tamaño de la ventana de contexto",
    "module_llama_cpp_config_context_window_input_accessible_description":
        "Establece el número de tokens que el modelo considera para generar respuestas.\n"
        "Controla cuánto contexto previo se utiliza.",

    "module_llama_cpp_chat_formats_label": "Formatos de chat",
    "module_llama_cpp_chat_formats_combo_placeholder_text": "Seleccione un formato de chat",
    "module_llama_cpp_chat_formats_combo_accessible_description":
        "Menú desplegable para seleccionar el formato utilizado para las conversaciones del modelo.",

    "module_llama_cpp_config_system_prompt_label": "Prompt del sistema",
    "module_llama_cpp_config_system_prompt_edit_placeholder_text": "Ingrese el texto del prompt del sistema",
    "module_llama_cpp_config_system_prompt_edit_accessible_description":
        "Campo de texto para introducir los prompts del sistema que guían las respuestas del modelo.",

    "module_llama_cpp_config_response_temperature_label": "Temperatura de respuesta: {temperature}",
    "module_llama_cpp_config_response_temperature_input_accessible_description":
        "Ajusta la aleatoriedad de las respuestas del modelo. Valores más altos producen resultados más variados,\n"
        "mientras que valores más bajos resultan en respuestas más predecibles.",

    "module_llama_cpp_config_response_max_tokens_label": "Máximos tokens por respuesta",
    "module_llama_cpp_config_response_max_tokens_input_accessible_description":
        "Limita el número de tokens en las respuestas del modelo hasta el límite actual de la ventana de contexto.\n"
        "Un valor de cero asume la capacidad de la ventana de contexto.",

    "module_llama_cpp_config_prompt_history_size_label": "Tamaño del historial de prompts",
    "module_llama_cpp_config_prompt_history_size_input_accessible_description":
        "Controla el número de entradas en el historial de prompts retenidas por el sistema para referencia.\n"
        "Un valor de cero permite entradas ilimitadas."
}



# Dutch lexemes main_menu.py
lexemes = {
    "group_file_label": "Bestand",
    "group_edit_label": "Bewerken",
    "group_tools_label": "Gereedschappen",
    "group_help_label": "Hulp",

    "actions_file_label_new_document": "Nieuw document",
    "actions_file_accessible_name_new_document": "Nieuw document",
    "actions_file_label_open": "Openen",
    "actions_file_accessible_name_open": "Document openen",
    "actions_file_label_save": "Opslaan",
    "actions_file_accessible_name_save": "Document opslaan",
    "actions_file_label_save_as": "Opslaan als",
    "actions_file_accessible_name_save_as": "Opslaan als documentkopie",
    "actions_file_label_settings": "Instellingen",
    "actions_file_accessible_name_settings": "App instellingen",
    "actions_file_label_reset_settings": "Instellingen resetten",
    "actions_file_accessible_name_reset_settings": "Instellingen resetten",
    "actions_file_label_exit": "Afsluiten",
    "actions_file_accessible_name_exit": "De app verlaten",

    "actions_edit_edit_mode": "Bewerken",
    "actions_edit_accessible_name_edit_mode": "Bestand bewerken",
    "actions_edit_source_mode": "Bron",
    "actions_edit_accessible_name_source_mode": "Broncode",
    "actions_edit_bold": "Vet",
    "actions_edit_accessible_name_bold": "Tekst in vetformaat",
    "actions_edit_italic": "Cursief",
    "actions_edit_accessible_name_italic": "Tekst in cursief formaat",
    "actions_edit_underline": "Onderstrepen",
    "actions_edit_accessible_name_underline": "Tekst onderstrepen",
    "actions_edit_strikethrough": "Doorhalen",
    "actions_edit_accessible_name_strikethrough": "Tekst doorhalen",
    "actions_edit_blockquote": "Citaat",
    "actions_edit_accessible_name_blockquote": "Tekst in citaatformaat",

    "actions_tools_ai_assistant": "AI Assistent",
    "actions_tools_accessible_name_ai_assistant": "AI Assistent starten",
    "actions_tools_color_picker": "Kleurkiezer",
    "actions_tools_accessible_name_color_picker": "Kies een tekstkleur",

    "actions_help_label_md_syntax": "Markdown-syntax",
    "actions_help_accessible_name_md_syntax": "Markdown-syntax",
    "actions_help_label_check_for_updates": "Controleer op updates",
    "actions_help_accessible_name_check_for_updates": "Controleer op updates",
    "actions_help_label_bug_report": "Bugrapport",
    "actions_help_accessible_name_bug_report": "Bugrapport indienen",
    "actions_help_label_about": "Over",
    "actions_help_accessible_name_about": "Over",
}



# English lexemes common.py
lexemes = {
    "app_title": "Notolog Editor",
    "app_title_with_sub": "{app_title} - {sub_title}",

    "tree_filter_accessible_desc": "File filter field",

    "menu_action_copy_file_path": "Copy file path",
    "menu_action_rename": "Rename",
    "menu_action_delete": "Delete",
    "menu_action_delete_completely": "Delete completely",
    "menu_action_restore": "Restore",
    "menu_action_create_new_dir": "Create a new directory",

    "dialog_file_rename_title": "Rename file",
    "dialog_file_rename_field_label": "Enter new file name",
    "dialog_file_rename_button_ok": "Rename",
    "dialog_file_rename_warning_exists": "File with the same name already exists",

    "dialog_file_delete_title": "Delete file",
    "dialog_file_delete_text": "Delete file \"{file_name}\"?",
    "dialog_file_delete_completely_title": "Delete file completely",
    "dialog_file_delete_completely_text": "Delete file \"{file_name}\" completely?",
    "dialog_file_delete_error": "Cannot delete file, error occurred",
    "dialog_file_delete_error_not_found": "File not found",

    "dialog_file_restore_title": "Restore file",
    "dialog_file_restore_text": "Restore file \"{file_name}\"?",
    "dialog_file_restore_error": "Cannot restore file, error occurred",
    "dialog_file_restore_warning_exists": "File with the name {file_name} already exists",

    "dialog_create_new_dir_title": "Create a new directory",
    "dialog_create_new_dir_label": "New directory name",
    "dialog_create_new_dir_input_placeholder_text": "Enter directory name",
    "dialog_create_new_dir_button_ok": "Create",
    "dialog_create_new_dir_button_cancel": "Cancel",
    "dialog_create_new_dir_warning_empty_title": "New directory name error",
    "dialog_create_new_dir_warning_empty_text": "Directory name cannot be empty",
    "dialog_create_new_dir_warning_too_long_title": "New directory name error",
    "dialog_create_new_dir_warning_too_long_text": "Directory name is too long; maximum {symbols} characters allowed!",
    "dialog_create_new_dir_error_existed": "Directory already exists",
    "dialog_create_new_dir_error": "Cannot create directory. Ensure the destination directory {base_dir} is writable",

    "dialog_message_box_title": "Message",
    "dialog_message_box_button_ok": "Close",

    "action_new_file_first_line_template_text": "New document",
    "action_open_file_dialog_caption": "Open File",
    "action_save_as_file_dialog_caption": "Save File",

    "dialog_save_empty_file_title": "Save empty file",
    "dialog_save_empty_file_text": "Allow to save the file with an empty content?",

    "dialog_encrypt_file_title": "Encrypt file",
    "dialog_encrypt_file_text": "Encrypt file \"{file_name}\"?",
    "encrypt_file_warning_file_is_already_encrypted": "The file is already encrypted!",
    "dialog_encrypt_file_rewrite_existing_title": "Rewrite existing file",
    "dialog_encrypt_file_rewrite_existing_text": "Rewrite existing file \"{file_path}\"?",

    "dialog_decrypt_file_title": "Decrypt file",
    "dialog_decrypt_file_text": "Decrypt file \"{file_name}\"?",
    "decrypt_file_warning_file_is_not_encrypted": "The file is not encrypted!",
    "dialog_decrypt_file_rewrite_existing_title": "Rewrite existing file",
    "dialog_decrypt_file_rewrite_existing_text": "Rewrite existing file \"{file_path}\"?",

    "dialog_encrypt_new_password_title": "New Password",
    "dialog_encrypt_new_password_label": "Password:",
    "dialog_encrypt_new_password_input_placeholder_text": "Enter New Password",
    "dialog_encrypt_new_password_hint_label": "Hint:",
    "dialog_encrypt_new_password_hint_label_description": "The hint is not encrypted and can be read from the file!"
                                                          "\nAvoid obvious hints that can be easily guessed, such as "
                                                          "birth dates."
                                                          "\nTry to use a reference that is not easily associated with you.",
    "dialog_encrypt_new_password_hint_input_placeholder_text": "Enter Hint (Optional)",
    "dialog_encrypt_new_password_button_ok": "OK",
    "dialog_encrypt_new_password_button_cancel": "Cancel",
    "dialog_encrypt_new_password_warning_empty_title": "Warning",
    "dialog_encrypt_new_password_warning_empty_text": "The password field cannot be empty!",
    "dialog_encrypt_new_password_warning_too_long_title": "Warning",
    "dialog_encrypt_new_password_warning_too_long_text": "The hint field is too long, maximum {symbols} characters!",

    "dialog_encrypt_password_title": "Enter Password",
    "dialog_encrypt_password_label": "Password:",
    "dialog_encrypt_password_input_placeholder_text": "Enter Password",
    "dialog_encrypt_password_hint_label": "Hint:",
    "dialog_encrypt_password_button_ok": "OK",
    "dialog_encrypt_password_button_cancel": "Cancel",

    "dialog_encrypt_password_reset_title": "Reset Encryption Password",
    "dialog_encrypt_password_reset_text": "Are you sure you want to reset the current encryption password?",
    "dialog_encrypt_password_reset_button_cancel": "Cancel",
    "dialog_encrypt_password_reset_button_yes": "Yes",

    "dialog_open_link_title": "Link",
    "dialog_open_link_text": "Open link \"{url}\" in a browser?",

    "dialog_reset_settings_title": "Reset settings?",
    "dialog_reset_settings_text":
        "All stored data in settings will be cleared, and the app will restart to apply changes.",

    "dialog_exit_unsaved_title": "Confirm Exit",
    "dialog_exit_unsaved_text": "The opened file '{file_name}' cannot be saved. Continue with exit?",

    "message_app_config_file_access": "Permission denied when accessing the app configuration file at {file_path}. "
        "Set the correct permissions to ensure proper operation.",

    "field_dir_path_dialog_caption": "Select Directory",
    "field_file_path_dialog_caption": "Select File",

    "load_file_encryption_password_mismatch": "Encryption password mismatch!",
    "load_file_encryption_password_incorrect": "Incorrect encryption password!",
    "load_file_none_content_error": "File cannot be loaded.",

    "action_new_file_error_occurred": "Cannot create file; an error occurred.\nCheck file system permissions.",
    "save_active_file_error_occurred": "Cannot save file; an error occurred.",

    "expandable_block_default_title": "More info...",
    "expandable_block_open_close_tags_mismatch_warning": "<details> block open/close tags mismatch",

    "dialog_color_picker_color_copied_to_the_clipboard": "Formatted text has been copied to the clipboard",

    "popup_about_title": "Application Info",
    "popup_about_app_name_description": "Python Markdown Editor",

    "popup_about_version": "Version",
    "popup_about_license": "License",
    "popup_about_website": "Website",
    "popup_about_repository": "GitHub",
    "popup_about_pypi": "PyPi",
    "popup_about_date": "Date",

    "update_helper_new_version_is_available": "A new version {latest_version} of the app is available",
    "update_helper_latest_version_installed": "The latest version of the app is installed",

    "network_connection_error_empty": "Cannot obtain response information",
    "network_connection_error_connection_or_dns":
        "Host not found. There may be an issue with the internet connection or DNS.",
    "network_connection_error_connection_refused":
        "Connection refused. The server might be down, or there may be network issues.",
    "network_connection_error_connection_timed_out": "Connection timed out. There may be network issues.",
    "network_connection_error_connection_404_error":
        "Connection 404 error. The requested page or resource is not found.",
    "network_connection_error_generic_with_status_code": "Request failed with status code: {status_code}",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Base Syntax Highlighter class tailored for Notolog.
- Functionality: This module contains shared functionality for subclasses like the MdHighlighter.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import Qt, QRegularExpression
from PySide6.QtGui import QTextDocument, QSyntaxHighlighter, QTextCharFormat
from PySide6.QtGui import QColor, QBrush, QFont

from typing import TYPE_CHECKING

from . import AppConfig
from . import ThemeHelper

from ..font_loader import FontLoader

import logging

if TYPE_CHECKING:
    from typing import Union  # noqa: F401
    from . import TextBlockData  # noqa: F401
    from PySide6.QtGui import QTextBlock, QTextBlockUserData  # noqa: F401


class MainHighlighter(QSyntaxHighlighter):
    """
    Syntax highlighter class
    """

    theme = {}
    # Keep it consistent with ini-file name, say 'main.ini', item prefix 'main_color_h1_text'
    theme_ini_prefix = 'main'

    """
    Elements order is matter!
    Say, i* first, b** second, bi*** third one-by-one to override prev token.
        i_, b__, bi___
        code `...`, code ```...```

    nth 0 is a whole string
    nth > 1 calculates as: start = end - length
    """
    re_rules = []

    def __init__(self, document: QTextDocument = None):
        super().__init__(document)

        self.logger = logging.getLogger('highlighter')

        # Get app's global font size
        self.font_size = AppConfig().get_font_size()

        # Walk rules to create QRegExp for each pattern
        self.rules = [(self.get_regex(pattern), nth, tag, group, duple, fmt, reckon)
                      for (pattern, nth, tag, group, duple, fmt, reckon) in self.re_rules]

        # Collect found tokens
        self.tokens = {}
        # Tokens which belong the same line
        self.line_tokens = {}

        # For debug purposes
        self.line_number = None
        self.line_number_log = None

        # Whether a group formatted or not
        self.formatted_group = {}

        # Indicate that the process is a re-highlighting process
        self.rehighlight_block = None

        self.current_block = None  # type: Union[QTextBlock, None]
        self.prev_block = None  # type: Union[QTextBlock, None]

        self.user_data = None  # type: Union[TextBlockData, QTextBlockUserData, None]
        self.prev_user_data = None  # type: Union[TextBlockData, QTextBlockUserData, None]

        self.theme_helper = ThemeHelper()

        self.override_colors()

    def __init_subclass__(cls):
        # Check the required methods are implemented
        if not hasattr(cls, 'highlightBlock'):
            raise NotImplementedError(f'Method highlightBlock() have to be implemented in subclass {cls.__name__}.')

    def override_colors(self):
        for item in self.theme:
            """
            Proof of concept method of how to override the colors set to highlighter's syntax.
            Consider overriding the whole map with styles like background pattern.
            """
            # Get color values from the theme helper
            # For example: 'md_color_h1_text', where the 'md' is the theme prefix, and the 'h1_text' is the item
            _color = self.theme_helper.get_color(f'{self.theme_ini_prefix}_color_{item}', css_format=True)
            _background_color = self.theme_helper.get_color(f'{self.theme_ini_prefix}_background_color_{item}')
            _background_color_inner = self.theme_helper.get_color(
                f'{self.theme_ini_prefix}_background_color_{item}_inner')
            self.logger.debug(
                f"Prefix[{self.theme_ini_prefix}] '{item}', color: '{_color}', bg color: '{_background_color}'")
            # At the moment all the colors set in string format, like 'red', 'darkCyan', etc.
            if _color and isinstance(_color, str):
                # Set up color no matter if it was set up or not
                self.theme[item]['color'] = _color
            # Background override
            if _background_color and isinstance(_background_color, str):
                if 'bg' in self.theme[item]:
                    if 'color' in self.theme[item]['bg']:
                        self.theme[item]['bg']['color'] = _background_color
                    elif isinstance(self.theme[item]['bg'], str):
                        self.theme[item]['bg'] = _background_color
                else:
                    # Set up background color no matter if it was set up or not
                    self.theme[item]['bg'] = _background_color
            # Inner color applied to elements within another element, such as Italic text within a Blockquote
            if _background_color_inner and isinstance(_background_color_inner, str):
                if 'bg_inner' in self.theme[item]:
                    if 'color' in self.theme[item]['bg_inner']:
                        self.theme[item]['bg_inner']['color'] = _background_color_inner
                    elif isinstance(self.theme[item]['bg_inner'], str):
                        self.theme[item]['bg_inner'] = _background_color_inner

    def get_regex(self, pattern) -> QRegularExpression:
        """
        Get either QRegularExpression or regular Python re
        """
        return QRegularExpression(pattern)

    #
    # Text block style format
    #
    # @staticmethod
    def cf(self, **kwargs):  # noqa: C901
        """
        Return a QTextCharFormat with style attributes

        Example of color names:
        https://doc.qt.io/qt-6/qcolor.html#predefined-colors
        """

        color = kwargs['color'] if 'color' in kwargs else ''
        style = kwargs['style'] if 'style' in kwargs else ''
        bg = kwargs['bg'] if 'bg' in kwargs else {}
        font_size_ratio = kwargs['font_size_ratio'] if 'font_size_ratio' in kwargs else 1.0

        if not (color or style or bg or font_size_ratio):
            return

        cformat = QTextCharFormat()

        # Foreground
        if color:
            color_obj = QColor(color)
            cformat.setForeground(color_obj)

        # Background
        if bg:
            bg_color = None  # type: Union[str, None]
            """
            Qt::BrushStyle https://doc.qt.io/qt-6/qt.html#BrushStyle-enum
            """
            bg_pattern = Qt.BrushStyle.SolidPattern
            if type(bg) is str:
                bg_color = bg  # type: ignore
            elif type(bg) is dict:
                if 'color' in bg:
                    bg_color = bg['color']
                if 'pattern' in bg and bg['pattern'] in Qt.BrushStyle:
                    bg_pattern = bg['pattern']
            # Color required
            if bg_color is not None:
                bg_color_obj = QColor(bg_color)
                """
                https://doc.qt.io/qt-6/qbrush.html
                """
                cformat.setBackground(QBrush(bg_color_obj, bg_pattern))

        if 'bold' in style:
            cformat.setFontWeight(QFont.Weight.Bold)
        if 'italic' in style:
            cformat.setFontItalic(True)
        if 'underline' in style:
            cformat.setFontUnderline(True)
        if 'strikethrough' in style:
            cformat.setFontStrikeOut(True)
        if 'monospace' in style:
            if loaded_monospace_font := FontLoader.get_monospace_font():
                monospace_font = QFont(loaded_monospace_font)
                monospace_font.setStyleHint(QFont.StyleHint.Monospace)
                # 100% means normal spacing, no extra spacing
                monospace_font.setLetterSpacing(QFont.SpacingType.PercentageSpacing, 100)
                cformat.setFont(monospace_font)

        # Explicitly set the font size after any potential font style changes
        if font_size_ratio and self.font_size:
            # The actual size is determined based on the base size set in the constructor
            cformat.setFontPointSize(int(self.font_size * font_size_ratio))

        return cformat

    def rehighlightBlock(self, block):
        self.logger.debug('Re-highlighting block data')

        # Leave as default
        self.rehighlight_block = True

        # Initial values
        self.tokens = {}
        self.line_tokens = {}
        self.line_number = None

        super().rehighlightBlock(block)

    def rehighlight(self):
        self.logger.debug('Re-highlighting the whole document')

        self.rehighlight_block = False

        # Initial values
        self.tokens = {}
        self.line_tokens = {}
        self.line_number = None

        super().rehighlight()

        self.rehighlight_block = True

    def get_open_close_token_map(self):
        return []

    def get_nl_closing_tokens(self):
        return []

    def highlightBlock(self, text_str):
        """
        Apply a syntax highlighting to each line of the text.
        https://doc.qt.io/qt-6/qsyntaxhighlighter.html#highlightBlock
        """
        pass

    def get_opened_group_token(self, group):
        for token_data in self.get_open_close_token_map():
            if (group == token_data['group']
                    and token_data['open'] in self.tokens
                    and self.tokens[token_data['open']]['o'] is True):
                return token_data['open']
        return None

    def is_any_formatted(self):
        return True if self.formatted_group else False

    def is_group_formatted(self, group):
        return True if group in self.formatted_group else False

    def set_formatted(self, group):
        # token group
        if group in self.formatted_group:
            self.formatted_group[group] += 1
        else:
            self.formatted_group[group] = 1

    def clear_formatted(self):
        self.formatted_group = {}



# French lexemes settings_dialog.py
lexemes = {
    "tab_module_llama_cpp_config": "Module llama.cpp",

    "module_llama_cpp_config_label": "Module llama.cpp",
    "module_llama_cpp_config_path_label": "Emplacement du modèle",
    "module_llama_cpp_config_path_input_placeholder_text": "Sélectionnez ou entrez le chemin du modèle",
    "module_llama_cpp_config_path_input_accessible_description":
        "Un champ de saisie avec un sélecteur pour spécifier le chemin du modèle local.\n"
        "Prend en charge les modèles au format GGUF, un format de fichier binaire optimisé\n"
        "pour stocker des modèles utilisés avec GGML et des exécuteurs basés sur GGML.",
    "module_llama_cpp_config_path_input_filter_text": "Fichiers GGUF",

    "module_llama_cpp_config_context_window_label": "Taille de la fenêtre de contexte",
    "module_llama_cpp_config_context_window_input_accessible_description":
        "Définit le nombre de jetons que le modèle considère pour générer des réponses.\n"
        "Contrôle la quantité de contexte préalable utilisée.",

    "module_llama_cpp_chat_formats_label": "Formats de chat",
    "module_llama_cpp_chat_formats_combo_placeholder_text": "Sélectionnez un format de chat",
    "module_llama_cpp_chat_formats_combo_accessible_description":
        "Menu déroulant pour sélectionner le format utilisé pour les conversations modèles.",

    "module_llama_cpp_config_system_prompt_label": "Invite de système",
    "module_llama_cpp_config_system_prompt_edit_placeholder_text": "Entrez le texte de l'invite système",
    "module_llama_cpp_config_system_prompt_edit_accessible_description":
        "Champ de texte pour entrer les invites de système qui guident les réponses du modèle.",

    "module_llama_cpp_config_response_temperature_label": "Température de réponse : {temperature}",
    "module_llama_cpp_config_response_temperature_input_accessible_description":
        "Ajuste l'aléatoire des réponses du modèle. Des valeurs plus élevées produisent des sorties plus variées,\n"
        "tandis que des valeurs plus basses résultent en des réponses plus prévisibles.",

    "module_llama_cpp_config_response_max_tokens_label": "Nombre maximal de jetons par réponse",
    "module_llama_cpp_config_response_max_tokens_input_accessible_description":
        "Limite le nombre de jetons dans les réponses du modèle jusqu'à la limite réelle de la fenêtre de contexte.\n"
        "Une valeur nulle suppose la capacité de la fenêtre de contexte.",

    "module_llama_cpp_config_prompt_history_size_label": "Taille de l'historique des invites",
    "module_llama_cpp_config_prompt_history_size_input_accessible_description":
        "Contrôle le nombre d'entrées dans l'historique des invites conservé par le système pour référence.\n"
        "Une valeur nulle permet un nombre illimité d'entrées."
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: This class is intended to help with the line numbers area on the left side of the main document editor.

Features:
  - Shows line/block numbers for better text recognition.
  - Highlights the current line where the cursor is located.

Note:
  - If used with PyQt6, ensure slots and signals are declared correctly, as there are some differences:
    `from PyQt6.QtCore import pyqtSlot as Slot`

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import Qt, QRect, QSize, Slot
from PySide6.QtWidgets import QWidget, QTextEdit, QPlainTextEdit
from PySide6.QtGui import QPainter, QColor, QTextFormat

from typing import TYPE_CHECKING, Union

from . import Settings
from . import ThemeHelper

from ..edit_widget import EditWidget

if TYPE_CHECKING:
    from PySide6.QtCore import QObject  # noqa: F401
    from PySide6.QtGui import QTextCharFormat  # noqa: F401


class LineNumbers(QWidget):
    def __init__(self, editor: Union[EditWidget, QPlainTextEdit] = None):
        QWidget.__init__(self, editor)

        self.editor = editor

        """
        # Alternatively it can be get from AppConfig:
        font = QFont()
        font.setPointSize(AppConfig().get_font_size())
        self.setFont(font)
        """
        self.setFont(self.editor.document().defaultFont())  # Set global font to the line area

        # Theme helper
        self.theme_helper = ThemeHelper()

        # Settings
        self.settings = Settings(parent=self)

        # Connect editor's change events to line numbers updates
        self.editor.updateRequest.connect(self.update_request)
        self.editor.blockCountChanged.connect(self.update_width)

        # The current method of mixing background colors does not yield clear results.
        # editor.cursorPositionChanged.connect(self.highlight_current_line)

        # Init object with direct updates
        self.update_numbers()

        # self.highlight_current_line()

    def sizeHint(self) -> QSize:
        return QSize(self.line_numbers_width(), 0)

    def get_numbers_area(self, area_block) -> QRect:
        """
        Calculates the area occupied by the line numbers widget for a given block, adjusted for
        the block's position within the document.

        For more information on QPainter usage, see: https://doc.qt.io/qt-6/qpainter.html#translate-2

        Args:
            area_block (QTextBlock): The text block for which to calculate the area,
                                     typically retrieved with self.editor.firstVisibleBlock().

        Returns:
            QRect: The calculated rectangular area that the line numbers widget occupies.
        """

        # Get the bounding rectangle adjusted for the block's position within the document,
        # which can include translation offsets:
        area_top = self.editor.blockBoundingGeometry(area_block).translated(self.editor.contentOffset()).top()

        # Calculate the height of the block's bounding rectangle (local coordinates)
        area_height = self.editor.blockBoundingRect(area_block).height()

        # Alternative approach (constant line height):
        # self.fontMetrics().height() * block.lineCount()

        # Return the computed QRect for the area
        return QRect(0, int(area_top) + 1, self.width(), int(area_height))

    def paintEvent(self, event) -> None:
        painter = QPainter(self)

        # Line number and background colors
        number_color = QColor(self.theme_helper.get_color('edit_widget_line_numbers_color'))
        number_background = QColor(self.theme_helper.get_color('edit_widget_line_numbers_background'))
        # Current line number and background colors
        current_number_color = QColor(self.theme_helper.get_color('edit_widget_line_numbers_active_number'))
        current_number_background = QColor(
            self.theme_helper.get_color('edit_widget_line_numbers_active_number_background'))

        # Get current line number (block) to highlight
        cursor = self.editor.textCursor()
        current_line_block_number = cursor.block().blockNumber()

        block = self.editor.firstVisibleBlock()
        block_number = block.blockNumber()

        # Calculate the line numbers area for the specified block
        numbers_area = self.get_numbers_area(block)

        # Get the editor's geometry (position and size)
        background_area = self.editor.geometry()
        background_area.adjust(1, 1, 0, -1)  # Reduce by 1 pixel on each corresponding side

        # Set the width of the background area to match the line numbers widget
        background_area.setWidth(self.width())

        # Fill the adjusted area with the background color
        painter.fillRect(background_area, number_background)

        while block.isValid() and numbers_area.top() <= event.rect().bottom():
            if numbers_area.bottom() >= event.rect().top():
                if current_line_block_number == block_number:
                    """
                    Highlight current line number
                    More info of how to fill rectangle area https://doc.qt.io/qt-6/qpainter.html#fillRect-4
                    """
                    painter.fillRect(numbers_area, current_number_background)
                    painter.setPen(current_number_color)
                else:
                    # Not selected number color
                    painter.fillRect(numbers_area, Qt.GlobalColor.transparent)
                    """
                    Also:
                    - painter.setPen(Qt.GlobalColor.darkGray)
                    - QColor(Qt.GlobalColor.green).lighter(179)
                    Global color enum https://doc.qt.io/qt-6/qt.html#GlobalColor-enum
                    It can be a color value or pre-set enum's value by name, like: Qt.GlobalColor.transparent
                    """
                    painter.setPen(number_color)

                number = str(block_number + 1)
                # Shift to the left to get the right padding from the overall padding
                painter.drawText(-2, numbers_area.top(), self.width(), self.fontMetrics().height(),
                                 Qt.AlignmentFlag.AlignRight, number)

            block = block.next()
            if block.isValid():
                block_number = block.blockNumber()
                numbers_area = self.get_numbers_area(block)

        painter.end()

    @Slot(QRect, int)
    def update_request(self, rect, dy) -> None:
        visible_rect = self.editor.viewport().rect()
        if dy:
            # Scroll the widget vertically by dy pixels
            self.scroll(0, dy)

        if rect.contains(visible_rect):
            # If the entire viewport is contained within rect, update rect area
            self.update(rect)  # Explicitly requests a repaint
            self.update_width()
        else:
            # Otherwise, update only the visible viewport area
            self.update(visible_rect)  # Explicitly requests a repaint
            # Previous approach:
            # self.update(0, rect.y(), self.width(), rect.height())

    @Slot(int)
    def update_width(self) -> None:
        self.editor.setViewportMargins(self.line_numbers_width(), 0, 0, 0)

    @Slot()
    def highlight_current_line(self) -> None:
        # The text edit widget is not intended to be a read only atm, but check it anyway to make it future-proof
        if not self.editor.isReadOnly():
            selection = QTextEdit.ExtraSelection()  # type: Union[QTextEdit.ExtraSelection, QTextCharFormat, None]
            if not selection:
                return
            selection.cursor = self.editor.textCursor()
            selection.cursor.clearSelection()
            # Set selection properties
            # line_color = QColor(Qt.GlobalColor.green).lighter(175)  # As an example of how to shift base color tone
            line_color = QColor(self.theme_helper.get_color('edit_widget_line_numbers_active_line_background'))
            _format = selection.format  # type: ignore
            _format.setBackground(line_color)
            _format.setProperty(QTextFormat.Property.FullWidthSelection, True)
            # extra_selections.append(selection)
            selection.format = _format
            self.editor.setExtraSelections([selection])

    def update_numbers(self) -> None:
        # Get element parent's contents size as the text edit widget may not be resized yet
        parent_obj = self.editor.parent()  # type: Union[QObject, QWidget]
        element_cr = parent_obj.contentsRect()
        # Update width first
        self.update_width()
        # Or it can be like this: height = self.editor.frameGeometry().height()
        self.setGeometry(
            QRect(element_cr.left(), element_cr.top(), self.line_numbers_width(), element_cr.height())
        )

    def line_numbers_width(self) -> int:
        """
        Get line space width
        @return: int value of space occupied by digits (incl. paddings)
        """
        if not self.visible():
            return 0

        digits = max(2, len(str(self.editor.blockCount())))
        # '5' is an overall value of left and right padding
        space = 5 + self.fontMetrics().horizontalAdvance("0") * digits
        return space

    def visible(self) -> bool:
        return self.settings.show_line_numbers



# Dutch lexemes toolbar.py
lexemes = {
    "actions_label_new_document": "Nieuw document",
    "actions_accessible_name_new_document": "Nieuw document",
    "actions_label_open": "Openen",
    "actions_accessible_name_open": "Document openen",
    "actions_label_save": "Opslaan",
    "actions_accessible_name_save": "Document opslaan",
    "actions_label_save_as": "Opslaan als",
    "actions_accessible_name_save_as": "Documentkopie opslaan",
    "actions_label_sync": "Synchroniseren",
    "actions_accessible_name_sync": "Document synchroniseren met externe opslag",
    "actions_label_edit": "Bewerken",
    "actions_accessible_name_edit": "Bestand bewerken",
    "actions_label_source": "Bron",
    "actions_accessible_name_source": "Broncode",
    "actions_label_encrypt": "Versleutelen",
    "actions_accessible_name_encrypt": "Bestand versleutelen",
    "actions_label_decrypt": "Decrypteren",
    "actions_accessible_name_decrypt": "Bestand decrypteren",
    "actions_label_bold": "Vet",
    "actions_accessible_name_bold": "Tekst vetgedrukt maken",
    "actions_label_italic": "Cursief",
    "actions_accessible_name_italic": "Tekst cursiveren",
    "actions_label_underline": "Onderstrepen",
    "actions_accessible_name_underline": "Tekst onderstrepen",
    "actions_label_strikethrough": "Doorhalen",
    "actions_accessible_name_strikethrough": "Tekst doorhalen",
    "actions_label_blockquote": "Citaat",
    "actions_accessible_name_blockquote": "Tekst als citaat opmaken",
    "actions_label_color_picker": "Kleurenkiezer",
    "actions_accessible_name_color_picker": "Tekstkleur kiezen",
    "actions_label_ai_assistant": "AI Assistent",
    "actions_accessible_name_ai_assistant": "AI Assistent starten",
    "actions_label_about": "Over",
    "actions_accessible_name_about": "Over de app",
    "actions_label_exit": "Afsluiten",
    "actions_accessible_name_exit": "De app afsluiten",
    "actions_label_settings": "Instellingen",
    "actions_accessible_name_settings": "App instellingen",

    "search_input_label": "Zoeken",
    "search_input_placeholder_text": "Zoek tekst in het document",
    "search_input_accessible_description": "Zoekveld",
    "search_buttons_label_clear": "Zoekopdracht wissen",
    "search_buttons_accessible_name_clear": "Zoekveld leegmaken",
    "search_buttons_label_prev": "Vorige zoeken",
    "search_buttons_accessible_name_prev": "Vorig zoekresultaat",
    "search_buttons_label_next": "Volgende zoeken",
    "search_buttons_accessible_name_next": "Volgend zoekresultaat",
    "search_case_sensitive_accessible_name": "Hoofdlettergevoelig",
    "search_case_sensitive_tooltip": "Zoeken met hoofdlettergevoeligheid",
}



# Italian lexemes common.py
lexemes = {
    # Common
    "module_ondevice_llm_name": "LLM su dispositivo",

    "module_ondevice_llm_model_exception": "Errore modello non trovato: {error_msg}",
    "module_ondevice_llm_task_exception": "Impossibile inizializzare il generatore di modelli: {error_msg}",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Extends QComboBox functionality to facilitate Enum data.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import Qt
from PySide6.QtGui import QColor
from PySide6.QtWidgets import QComboBox


class EnumComboBox(QComboBox):
    def __init__(self, enum_class):
        super().__init__()
        self._enum_class = enum_class

        # Sort the Enum members by value
        # sorted_members = sorted(enum_class, key=lambda x: x.value)

        # Load the Enum values
        for enum_value in enum_class:  # Or: sorted_members
            self.addItem(enum_value.value, userData=enum_value)

            # Add separator. It also adds extra index to the QComboBox
            # if hasattr(enum_value, 'is_default') and enum_value.is_default:
            #    self.insertSeparator(i + 1)

            # Check if the current item is a legacy item
            if hasattr(enum_value, 'legacy') and enum_value.legacy:
                # Get the index of the item we just added
                index = self.count() - 1
                # Set a custom style for legacy items (e.g., gray text color)
                self.setItemData(index, QColor('gray'), Qt.ItemDataRole.ForegroundRole)
                # Alternatively, disable the item if you don't want it selectable
                # self.model().item(index).setEnabled(False)

    @property
    def enum_class(self):
        return self._enum_class



# French lexemes ai_assistant.py
lexemes = {
    # Assistant IA
    "dialog_title": "Requête à l'IA",
    "dialog_prompt_input_placeholder_text": "Requête",
    "dialog_prompt_input_accessible_description": "Entrer une requête pour donner un contexte à l'assistant IA",
    "dialog_response_output_placeholder_text": "Données de réponse",
    "dialog_response_output_accessible_description": "Les données de réponse apparaîtront dans ce champ",
    "dialog_response_output_notice_empty_text": "Fournissez une saisie de texte pour obtenir une réponse d'environ...",

    "dialog_usage_module_label": "MODULE",
    "dialog_usage_model_label": "MODÈLE",
    "dialog_usage_tokens_label": "JETONS",
    "dialog_usage_tokens_prompt": "invite : {tokens}",
    "dialog_usage_tokens_answer": "réponse : {tokens}",
    "dialog_usage_tokens_total": "total : {tokens}",

    "dialog_button_send_request": "Envoyer la requête",
    "dialog_button_stop_request": "Arrêter",
    "dialog_button_save_history": "Enregistrer l'historique du dialogue",

    "dialog_message_copy_tooltip": "Copier le texte",
    "dialog_message_copied_tooltip": "Copié",

    "dialog_error_loading_model": "Erreur lors du chargement du modèle personnalisé",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Manages text editor state, such as View and Edit modes, content source, and data encryption status.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

import logging

from enum import Enum

# from PyQt6.QtCore import pyqtSignal as Signal, QObject
from PySide6.QtCore import Signal, QObject


class Mode(Enum):
    VIEW = 1  # Default
    EDIT = 2
    SOURCE = 3  # View result source code


class Source(Enum):
    MARKDOWN = 1
    HTML = 2  # Default


class Encryption(Enum):
    PLAIN = 0  # Default
    ENCRYPTED = 1


class EditorState(QObject):
    value_changed = Signal(object)

    def __init__(self, parent=None):
        super(EditorState, self).__init__(parent)

        self.logger = logging.getLogger('editor_state')

        self._mode = None
        self._source = None
        self._encryption = None

        # Previous values for switching back purposes
        self._prev_mode = None
        self._prev_source = None

        # Dialog answer of either to allow to save an empty file or not
        self._allow_save_empty = None

    @property
    def mode(self):
        return self._mode

    @property
    def prev_mode(self):
        return self._prev_mode

    @property
    def source(self):
        return self._source

    @property
    def prev_source(self):
        return self._prev_source

    @property
    def encryption(self):
        return self._encryption

    @property
    def allow_save_empty(self):
        return self._allow_save_empty

    @mode.setter
    def mode(self, value):
        self._prev_mode = self._mode  # Save previous state
        self._mode = value
        """
        c - context
        v - value
        pv - previous value
        """
        self.value_changed.emit({'c': Mode, 'v': value, 'pv': self._prev_mode})  # type: ignore

    @source.setter
    def source(self, value):
        self._prev_source = self._source  # Save previous state
        self._source = value
        self.value_changed.emit({'c': Source, 'v': value, 'pv': self._prev_source})  # type: ignore

    @encryption.setter
    def encryption(self, value):
        self._encryption = value
        self.value_changed.emit({'c': Encryption, 'v': value})  # type: ignore

    @allow_save_empty.setter
    def allow_save_empty(self, value):
        self._allow_save_empty = value

    def refresh(self):
        # To refresh UI upon settings update or at similar situations
        self.mode = self._mode
        self.source = self._source
        self.encryption = self._encryption

    def allow_save_empty_reset(self):
        # Allows to show dialog again
        self._allow_save_empty = None

    def toggle_mode(self):
        """
        Switch between VIEW and EDIT modes to maintain actual state.
        """

        self.logger.debug('Toggle mode %s' % self.mode)

        """
        Switch by this scheme:
        * Mode.SOURCE > Previous mode
        * Mode.EDIT > Mode.VIEW
        * Mode.VIEW > Mode.EDIT

        Mode SOURCE uses the same UI to show the data as VIEW mode.
        """
        if self.mode == Mode.SOURCE:
            """
            Remember the previous state whether it's the mode or source is only relevant for the changed parameter.
            For example: the actual mode is SOURCE, and the previous mode is VIEW. Hence, the parameter was changed.
            However, the previous source MARKDOWN may be unrelated to the previous mode VIEW,
            because of hasn't been changed after the mode switching.
            """
            self.mode = (self.prev_mode if self.prev_mode and self.prev_mode != Mode.SOURCE
                         else (Mode.EDIT if self.source == Source.MARKDOWN else Mode.VIEW))
        elif self.mode == Mode.VIEW:
            self.mode = Mode.EDIT
            self.source = Source.MARKDOWN
        else:
            self.mode = Mode.VIEW
            self.source = Source.HTML
            # To do not count previous answer if edit mode was switched once
            self.allow_save_empty_reset()

        self.logger.debug("Toggled mode to '%s', source to '%s'; previous states are '%s' and '%s' correspondingly"
                          % (self.mode, self.source, self.prev_mode, self.prev_source))



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from notolog.encrypt.enc_password import EncPassword

import pytest


class TestEncPassword:

    @pytest.fixture(scope="function", autouse=True)
    def test_obj_enc_password(self, request):

        # Retrieve parameter values from the test request.
        password, hint = request.param

        _enc_password = EncPassword()
        _enc_password.password = password
        _enc_password.hint = hint

        yield _enc_password

    @pytest.fixture(scope="function")
    def test_exp_params_fixture(self, request):
        # Retrieve parameter values from the test request.
        password, hint = request.param

        yield password, hint

    @pytest.mark.parametrize(
        "test_obj_enc_password, test_exp_params_fixture",
        [
            ((None, None), (None, None)),
            (('some>pass>here', None), ('some>pass>here', None)),
            ((None, 'some-hint-here'), (None, 'some-hint-here')),
            (('some>pass>here', 'some-hint-here'), ('some>pass>here', 'some-hint-here')),
        ],
        indirect=True
    )
    def test_enc_password_init(self, test_obj_enc_password, test_exp_params_fixture):
        exp_password, exp_hint = test_exp_params_fixture

        assert hasattr(test_obj_enc_password, 'password')
        assert hasattr(test_obj_enc_password, 'hint')
        # Password and salt are expectedly set
        assert test_obj_enc_password.password == exp_password
        assert test_obj_enc_password.hint == exp_hint

    @pytest.fixture(scope="function")
    def test_exp_is_valid_fixture(self, request):
        # Get the parameter value from the request
        param_value = request.param

        yield param_value

    @pytest.mark.parametrize(
        "test_obj_enc_password, test_exp_is_valid_fixture",
        [
            ((None, None), False),
            (('>pass>', None), False),
            (('>pass>', 'some-hint-here'), False),
            (('01234567', None), True),
            (('some>pass>here', None), True),
            (('some>pass>here', 'some-hint-here'), True),
        ],
        indirect=True
    )
    def test_enc_password_is_valid(self, test_obj_enc_password: EncPassword, test_exp_is_valid_fixture):
        assert test_obj_enc_password.is_valid() == test_exp_is_valid_fixture



# Georgian lexemes settings_dialog.py
lexemes = {
    # პარამეტრების დიალოგი
    "window_title": "პარამეტრები",

    "button_close": "დახურვა",

    "tab_general": "ძირითადი",
    "tab_editor_config": "რედაქტორი",
    "tab_viewer_config": "დამკვრელი",
    "tab_ai_config": "AI კონფიგურაცია",

    "general_app_config_label": "აპლიკაციის კონფიგურაცია",
    "general_app_language_label": "ენა",
    "general_app_language_combo_placeholder_text": "აირჩიეთ ენა",
    "general_app_language_combo_accessible_description": "აპლიკაციის ინტერფეისის ენა",
    "general_app_theme_label": "თემა",
    "general_app_theme_combo_placeholder_text": "აირჩიეთ თემა",
    "general_app_theme_combo_accessible_description": "აპლიკაციის ინტერფეისის თემა",
    "general_app_main_menu_label": "მთავარი მენიუ",
    "general_app_main_menu_checkbox": "მთავარი მენიუს ჩვენება",
    "general_app_main_menu_checkbox_accessible_description": "აჩვენეთ აპლიკაციის მთავარი ჩამოშლილი მენიუ",
    "general_app_font_size_label": "შრიფტის ზომა: {size}pt",
    "general_app_font_size_slider_accessible_description": "შრიფტის გლობალური ზომის რეგულირება",

    "general_statusbar_label": "სტატუსის ზოლი",
    "general_statusbar_show_global_cursor_position_checkbox": "მსოფლიო კურსორის პოზიციის ჩვენება",
    "general_statusbar_show_global_cursor_position_checkbox_accessible_description":
        "აჩვენეთ კურსორის მსოფლიო პოზიცია სტატუსის ზოლში",

    "editor_config_label": "რედაქტორის კონფიგურაცია",
    "editor_config_show_line_numbers_checkbox": "ხაზების ნომრების ჩვენება",
    "editor_config_show_line_numbers_checkbox_accessible_description": "ხაზების ნომრების ჩვენება რედაქტორში",

    "viewer_config_label": "დამკვრელის კონფიგურაცია",
    "viewer_config_process_emojis_checkbox": "ტექსტური ემოჯის გრაფიკულად გარდაქმნა",
    "viewer_config_process_emojis_checkbox_accessible_description": "ტექსტური ემოჯების გრაფიკული წარმოჩენა",
    "viewer_config_highlight_todos_checkbox": "TODO-ების გამოყოფა",
    "viewer_config_highlight_todos_checkbox_accessible_description": "TODO ტეგების გამოყოფა ტექსტში",
    "viewer_config_open_link_confirmation_checkbox": "ბმულების გახსნის დასტური",
    "viewer_config_open_link_confirmation_checkbox_accessible_description":
        "ბმულების გახსნისას დასტურის მოთხოვნა",
    "viewer_config_save_resources_checkbox": "გარე სურათების ავტომატური შენახვა დისკზე",
    "viewer_config_save_resources_checkbox_accessible_description":
        "გარე სურათების ასლების ავტომატურად შენახვა დისკზე ოფლაინ წვდომისთვის.",

    "ai_config_inference_module_label": "ინფერენციის მოდული",
    "ai_config_inference_module_names_combo_label": "აქტიური ინფერენციის მოდული",
    "ai_config_inference_module_names_combo_placeholder_text": "აირჩიეთ მოდული",
    "ai_config_inference_module_names_combo_accessible_description":
        "აირჩიეთ AI ასისტენტთან მუშაობადი არსებული AI ინფერენციის მოდულებიდან.\n"
        "ოფციები მოიცავს ლოკალური დიდი ენის მოდელებს (LLM) რეალური დროის დამუშავებით,\n"
        "ან API-ზე დაყრდნობითი ფუნქციონალობით.",

    "ai_config_base_label": "ძირითადი პარამეტრები",
    "ai_config_multi_turn_dialogue_checkbox": "მრავალშრიანი დიალოგი საუბრის მეხსიერებით",
    "ai_config_multi_turn_dialogue_checkbox_accessible_description":
        "ჩართეთ მრავალშრიანი დიალოგი, რომელიც ინახავს წინა მოთხოვნას საუბრის მეხსიერებისთვის.\n"
        "გამორთვის შემთხვევაში, მხოლოდ ახალი შეტყობინება და სისტემის პრომპტი ახდენენ გავლენას პასუხზე.",
    "ai_config_convert_to_md_checkbox": "შედეგის Markdown-ში კონვერტაცია",
    "ai_config_convert_to_md_checkbox_accessible_description":
        "გადაიყვანეთ გამოსახული შეტყობინება Markdown ფორმატში.",
}
